package Nawt::DateTime;
# $Id: DateTime.pm,v 1.3 2005/04/11 18:03:45 ramirez Exp $
use Date::Calc;
use Exporter;
use strict;
use Nawt::Session;

our @ISA = qw(Exporter);
our @EXPORT = qw(
	&localtime 
	&addToTime 
	&addToDate 
	&epochToHuman 
	&epochToSet 
	&humanToEpoch 
	&setToEpoch
	&secondsToInterval2 
	&monthStartEnd);

#-------------------------------------------------------------------
# addToDate ( epoch [ , years, months, days ] ) Returns an epoch date with the amount of time added.
sub addToDate {
	my ($year,$month,$day, $hour,$min,$sec, $newDate);
	($year,$month,$day, $hour,$min,$sec) = Date::Calc::Time_to_Date($_[0]);
	($year,$month,$day) = Date::Calc::Add_Delta_YMD($year,$month,$day, $_[1],$_[2],$_[3]);
	$newDate = Date::Calc::Date_to_Time($year,$month,$day, $hour,$min,$sec);
	return $newDate;
}

#-------------------------------------------------------------------
# addToTime ( epoch [ , hours, minutes, seconds ] ) Returns an epoch date with the amount of time added.
sub addToTime {
        my ($year,$month,$day, $hour,$min,$sec, $newDate);
        ($year,$month,$day, $hour,$min,$sec) = Date::Calc::Time_to_Date($_[0]);
        ($year,$month,$day, $hour,$min,$sec) = Date::Calc::Add_Delta_DHMS($year,$month,$day,$hour,$min,$sec,0,$_[1],$_[2],$_[3]);
        $newDate = Date::Calc::Date_to_Time($year,$month,$day, $hour,$min,$sec);
        return $newDate;
}

#-------------------------------------------------------------------
# dayStartEnd ( epoch ) Returns the epoch dates for the start and end of the day.
sub dayStartEnd {
        my ($year,$month,$day, $hour,$min,$sec, $start, $end);
        ($year,$month,$day, $hour,$min,$sec) = Date::Calc::Time_to_Date($_[0]);
        $start = Date::Calc::Date_to_Time($year,$month,$day,0,0,0);
        $end = Date::Calc::Date_to_Time($year,$month,$day,23,59,59);
        return ($start, $end);
}

#-------------------------------------------------------------------
# epochToHuman ( [ epoch, format ] ) Returns a formated date string.
#  %% = % (percent) symbol.
#  %c = The calendar month name.
#  %C = The calendar month name abbreviated to 3 characters and represented in English.
#  %d = A two digit day.
#  %D = A variable digit day.
#  %h = A two digit hour (on a 12 hour clock).
#  %H = A variable digit hour (on a 12 hour clock).
#  %j = A two digit hour (on a 24 hour clock).
#  %J = A variable digit hour (on a 24 hour clock).
#  %m = A two digit month.
#  %M = A variable digit month.
#  %n = A two digit minute.
#  %o = Offset from GMT represented as an integer.
#  %O = Offset from GMT represented in four digit form with a sign. Example: -0600
#  %p = A lower-case am/pm.
#  %P = An upper-case AM/PM.
#  %s = A two digit second.
#  %w = Day of the week. 
#  %W = Day of the week abbreviated to 3 characters and represented in English. 
#  %y = A four digit year.
#  %Y = A two digit year. 
#  %z = The current user's date format preference.
#  %Z = The current user's time format preference.
sub epochToHuman {
	my ($offset, $temp, $hour12, $value, $output);
	$offset = $session{config}{timeOffset} || 0;
	$offset = $offset*3600;
	$temp = int($_[0]) || time();
	$temp = $temp+$offset;
	my ($year,$month,$day,$hour,$min,$sec) = Date::Calc::Time_to_Date($temp);
	$output = $_[1] || "%z %Z";
  #---GMT Offsets
	$temp = $session{config}{timeOffset}*100;
	$temp = sprintf('%+05d',$temp);
	$output =~ s/\%O/$temp/g;
	$temp = $session{config}{timeOffset}+0;
	$output =~ s/\%o/$temp/g;
  #---dealing with percent symbol
	$output =~ s/\%\%/\%/g;
  #---date format preference
	$temp = $session{user}{dateFormat} || '%M/%D/%y';
	$output =~ s/\%z/$temp/g;
  #---time format preference
	$temp = $session{user}{timeFormat} || '%H:%n %p';
	$output =~ s/\%Z/$temp/g;
  #---year stuff
	$output =~ s/\%y/$year/g;
	$value = substr($year,2,2);
	$output =~ s/\%Y/$value/g;
  #---month stuff
	$value = sprintf("%02d",$month);
	$output =~ s/\%m/$value/g;
	$output =~ s/\%M/$month/g;
	if ($output =~ /\%c/) {
		$temp = getMonthName($month);
		$output =~ s/\%c/$temp/g;
	}
	if ($output =~ /\%C/) {
		$temp = substr(Date::Calc::Month_to_Text($month),0,3);
		$output =~ s/\%C/$temp/g;
	}
  #---day stuff
	$value = sprintf("%02d",$day);
	$output =~ s/\%d/$value/g;
	$output =~ s/\%D/$day/g;
	if ($output =~ /\%w/) {
		$temp = getDayName(Date::Calc::Day_of_Week($year,$month,$day));
		$output =~ s/\%w/$temp/g;
	}
	if ($output =~ /%W/) {
		$temp = Date::Calc::Day_of_Week_Abbreviation(Date::Calc::Day_of_Week($year,$month,$day));
		$output =~ s/\%W/$temp/g;
	}
  #---hour stuff
	$hour12 = $hour;
	if ($hour12 > 12) {
		$hour12 = $hour12 - 12;
	}	
	if ($hour12 == 0) {
		$hour12 = 12;
	}
	$value = sprintf("%02d",$hour12);
	$output =~ s/\%h/$value/g;
	$output =~ s/\%H/$hour12/g;
	$value = sprintf("%02d",$hour);
	$output =~ s/\%j/$value/g;
	$output =~ s/\%J/$hour/g;
	if ($hour > 11) {
		$output =~ s/\%p/pm/g;
		$output =~ s/\%P/PM/g;
	} else {
		$output =~ s/\%p/am/g;
		$output =~ s/\%P/AM/g;
	}
  #---minute stuff
	$value = sprintf("%02d",$min);
	$output =~ s/\%n/$value/g;
  #---second stuff
	$value = sprintf("%02d",$sec);
	$output =~ s/\%s/$value/g;
	return $output;
}

#-------------------------------------------------------------------
# epochToSet ( epoch )Returns a set date in the format of MM/DD/YYYY. 
sub epochToSet {
	return epochToHuman($_[0],"%m/%d/%y");
}

#-------------------------------------------------------------------
# getMonthName ( month ) Returns a string containing the calendar month name in the language of the current user.
sub getMonthName {
		my $result;
        if ($_[0] == 1) {
                $result = "January";
        } elsif ($_[0] == 2) {
                $result = "February";
        } elsif ($_[0] == 3) {
                $result = "March";
        } elsif ($_[0] == 4) {
                $result = "April";
        } elsif ($_[0] == 5) {
                $result = "May";
        } elsif ($_[0] == 6) {
                $result = "June";
        } elsif ($_[0] == 7) {
                $result = "July";
        } elsif ($_[0] == 8) {
                $result = "August";
        } elsif ($_[0] == 9) {
                $result = "September";
        } elsif ($_[0] == 10) {
                $result = "October";
        } elsif ($_[0] == 11) {
                $result = "November";
        } elsif ($_[0] == 12) {
                $result = "December";
        }
		if (defined $_[1]) {
			$result = substr($result, 0, $_[1]);
		}
		return $result;
}

#-------------------------------------------------------------------
# getDayName ( day ) Returns a string containing the weekday name. 
sub getDayName {
	my ($result, $day);
		$day = $_[0];
        if ($day == 7) {
                $result = "";
        } elsif ($day == 1) {
                $result = "";
        } elsif ($day == 2) {
                $result = "";
        } elsif ($day == 3) {
                $result = "";
        } elsif ($day == 4) {
                $result = "";
        } elsif ($day == 5) {
                $result = "";
        } elsif ($day == 6) {
                $result = "";
        }
		if (defined $_[1]) {
			$result = substr($result, 0, $_[1]);
		}
		return $result;
}

#-------------------------------------------------------------------
# getSecondsFromEpoch ( epoch ) Calculates the number of seconds into the day of an epoch date the epoch datestamp is.
sub getSecondsFromEpoch {
	return timeToSeconds(epochToHuman($_[0],"%j:%n:%s"));
}



#-------------------------------------------------------------------
# humanToEpoch ( date ) Returns an epoch date derived from the human date. YYYY-MM-DD HH:MM:SS 
sub humanToEpoch {
	my (@temp, $dateString, $timeString, $output, @date);
	($dateString,$timeString) = split(/ /,$_[0]);
	@temp = split(/-/,$dateString);
	$date[0] = int($temp[0]);
	$date[1] = int($temp[1]);
	$date[2] = int($temp[2]);
	@temp = split(/:/,$timeString);
	$date[3] = int($temp[0]);
	$date[4] = int($temp[1]);
	$date[5] = int($temp[2]);
	$output = Date::Calc::Date_to_Time(@date);
	return $output;
} 

#-------------------------------------------------------------------
# intervalToSeconds ( interval, units ) Returns the number of seconds derived from the interval. A string which represents the units of the interval. The string must be 'years', 'months', 'weeks', 'days', 'hours', 'minutes', or 'seconds'. 
sub intervalToSeconds {
	if ($_[1] eq "years") {
		return ($_[0]*31536000);
	} elsif ($_[1] eq "months") {
		return ($_[0]*2592000);
        } elsif ($_[1] eq "weeks") {
                return ($_[0]*604800);
        } elsif ($_[1] eq "days") {
                return ($_[0]*86400);
        } elsif ($_[1] eq "hours") {
                return ($_[0]*3600);
        } elsif ($_[1] eq "minutes") {
                return ($_[0]*60);
        } else {
                return $_[0];
	} 
}

#-------------------------------------------------------------------
# localtime ( [ epoch ] ) Returns an array of time elements. 
# The elements are: years, months, days, hours, minutes, seconds, day of year, day of week, daylight savings time.
sub localtime {
	return Date::Calc::Localtime($_[0]||Nawt::DateTime::time());
}

#-------------------------------------------------------------------
# monthCount ( startEpoch, endEpoch ) Returns the number of months between the start and end dates (inclusive).
sub monthCount {
	my ($start, $end) = @_;
	my @delta = Date::Calc::Delta_YMDHMS( Date::Calc::Time_to_Date($start), Date::Calc::Time_to_Date($end));
	my $change = (($delta[0]*12)+$delta[1])+1;
	return $change;
}

#-------------------------------------------------------------------
# monthStartEnd ( epoch ) Returns the epoch dates for the start and end of the month.
sub monthStartEnd {
	my ($year,$month,$day, $hour,$min,$sec, $start, $end);
	($year,$month,$day, $hour,$min,$sec) = Date::Calc::Time_to_Date($_[0]);
	$start = Date::Calc::Date_to_Time($year,$month,1,0,0,0)-$session{config}{timeOffset}*3600;
	($year,$month,$day, $hour,$min,$sec) = Date::Calc::Time_to_Date(addToDate($_[0],0,1,0));
	$end = Date::Calc::Date_to_Time($year,$month,1,0,0,0)-1;
	return ($start, $end);
}

#-------------------------------------------------------------------
sub monthDayCount {
	my ($t1, $t2) = monthStartEnd($_[0]);
    return round(($t2-$t1)/86400);
}

#-------------------------------------------------------------------
# secondsToInterval ( seconds ) Returns an interval and units derived the number of seconds.
sub secondsToInterval {
	my ($interval, $units);
	if ($_[0] >= 31536000) {
		$interval = round($_[0]/31536000);
		$units = "years";
	} elsif ($_[0] >= 2592000) {
        $interval = round($_[0]/2592000);
        $units = "months";
	} elsif ($_[0] >= 604800) {
        $interval = round($_[0]/604800);
        $units = "weeks";
	} elsif ($_[0] >= 86400) {
        $interval = round($_[0]/86400);
        $units = "days";
    } elsif ($_[0] >= 3600) {
        $interval = round($_[0]/3600);
        $units = "hours";
    } elsif ($_[0] >= 60) {
        $interval = round($_[0]/60);
        $units = "minutes";
    } else {
        $interval = $_[0];
        $units = "seconds";
	}
	return ($interval, $units);
}


#-------------------------------------------------------------------
# secondsToInterval ( seconds ) Returns an interval and units derived the number of seconds.
sub secondsToInterval2 {
	my ($interval, $units);
	if ($_[0] >= 31536000) {
		$interval = round($_[0]/31536000);
		$units = "years";
	} elsif ($_[0] >= 2592000) {
                $interval = round($_[0]/2592000);
                $units = "months";
	} elsif ($_[0] >= 604800) {
                $interval = round($_[0]/604800);
                $units = "weeks";
	} elsif ($_[0] >= 86400) {
                $interval = round($_[0]/86400);
                $units = "days";
        } elsif ($_[0] >= 3600) {
                $interval = round($_[0]/3600);
                $units = "hours";
        } elsif ($_[0] >= 60) {
                $interval = round($_[0]/60);
                $units = "minutes";
        } else {
                $interval = $_[0];
                $units = "seconds";
	}
	return $interval." ".$units;
}

#-------------------------------------------------------------------
# secondsToTime ( seconds ) Returns a time string of the format HH::MM::SS on a 24 hour clock. See also timeToSeconds().
sub secondsToTime {
	my $seconds = $_[0];
	my $timeString = sprintf("%02d",int($seconds / 3600)).":";
	$seconds = $seconds % 3600;	
	$timeString .= sprintf("%02d",int($seconds / 60)).":";
	$seconds = $seconds % 60;
	$timeString .= sprintf("%02d",$seconds);
	return $timeString;
}


#-------------------------------------------------------------------
# setToEpoch ( set ) Returns an epoch date. A string in the format of MM/DD/YYYY.
sub setToEpoch {
	my @date = Date::Calc::Time_to_Date(Nawt::DateTime::time());
 	my ($month, $day, $year) = split(/\//,$_[0]);
	if (int($year) < 2038 && int($year) > 1969) {
		$year = int($year);
	} else {
		$year = $date[0];
	}
        if (int($month) < 13 && int($month) > 0) {
                $month = int($month);
        } else {
                $month = $date[1]++;
        }
        if (int($day) < 32 && int($day) > 0) {
                $day = int($day);
        } else {
                $day = $date[2];
        }
	return Date::Calc::Date_to_Time($year,$month,$day,0,0,0);
}

#-------------------------------------------------------------------
# time() Returns an epoch date for now.
sub time {
	return Date::Calc::Date_to_Time(Date::Calc::Today_and_Now(1));
}

#-------------------------------------------------------------------
# timeToSeconds ( timeString ) Returns the seconds since 00:00:00 on a 24 hour clock.
sub timeToSeconds {
	my ($hour,$min,$sec) = split(/:/,$_[0]);
	return ($hour*3600+$min*60+$sec);
}


1;
