package Nawt::Operation::Utility;
# $Id: Utility.pm,v 1.11 2005/04/20 13:01:12 ramirez Exp $

use strict;
use Exporter;
use Nawt::SQL;
use Nawt::Session;
use Nawt::Privilegies;
use Nawt::DateTime;
use Nawt::Theme;
use Nawt::Navigation;
	
our @ISA = qw( Exporter ); 
our @EXPORT = qw(
	&www_showNetamsConfig 
	&www_createPerflog 
	&www_showNawtSessions 
	&www_closeNawtSession 
	&www_showNetamsLog
	&www_showPingUnits );


#--------------------------------------------------------------------
sub www_closeNawtSession {
	checkAdmin();
	my ($id);
	$id = $session{form}{id};
	Nawt::SQL->write("DELETE FROM nawt_session WHERE sessionId LIKE ".quote($id));
	return Nawt::Theme::getRedirect("?op=showNawtSessions");
}

#--------------------------------------------------------------------
sub www_showPingUnits {
	checkAdmin();
	my ($result,%units,%sql,%html,$row);
	%sql = Nawt::SQL->buildHash("SELECT unit_oid,message FROM nawt_ping WHERE status=1");
	%units = $session{netams}->getUnit();
	$session{page}{title} = "Ping Units";

	foreach (keys %sql) {
		my $oid = uc sprintf ("%6.6x",$_);
		next unless $session{netams}->hasUnit($oid);
		$row++;
		if ($row % 2) { $html{rowclass} = "row0"; } else { $html{rowclass} = "row1"; }
		$html{list} .= qq~<tr class=$html{rowclass}><td width=16><a href="?op=editUnit&oid=$oid"><img src="$session{config}{extrasURL}/images/icons/$units{$oid}{type}.gif"></a></td>
				<td><a href="?op=editUnit&oid=$oid">$units{$oid}{name}</a><div class=small>$units{$oid}{description}</div></td>
				<td>$sql{$_}</td>
			</tr>~;
	}
	
	$result = qq~
	<table class=adminlist>
	<tr class=boxhead><td colspan=2>Unit</td><td>Dead IPs / Message</td></tr>
	$html{list}</table>
	<p><div>    <b>./nawt/sbin/ping_units.pl</b>        .</div>
	~;
	return $result;
}

#--------------------------------------------------------------------
sub www_showNawtSessions {
	checkAdmin();
	my ($result, %html, @list, %main, %user, $row);
	$session{page}{title} = "NaWT active sessions";
	%main = $session{netams}->getService('main');
	%user = %{$main{users}};
	@list = Nawt::SQL->buildArray("SELECT sessionId FROM nawt_session ORDER BY userId DESC, isAdmin DESC");
	foreach (@list) {
		my %item = Nawt::SQL->quickHash("SELECT * FROM nawt_session WHERE sessionId LIKE '$_' LIMIT 1");
		my $lastact = epochToHuman($item{lastPageView},"%y-%m-%d %j:%n")." (".secondsToInterval2(time-$item{lastPageView})." ago)";
		my $expires = epochToHuman($item{expires},"%y-%m-%d %j:%n")." (in ".secondsToInterval2($item{expires}-time).")";
		if ($row % 2) { $html{rowclass} = "row0"; } else { $html{rowclass} = "row1"; }
		if ($item{isAdmin} and $item{userId} =~ m/^\w{6}$/) {
			$html{admins} .= qq~<tr class=$html{rowclass}>
				<td width=16><img src="$session{config}{extrasURL}/images/user.gif"></td>
				<td><a href="?op=editUser&oid=$item{userId}">$user{$item{userId}}{name} (admin)</a></td>
				<td>$item{lastIP}</td>
				<td>$lastact</td>
				<td>$expires</td>
				<td><a href="?op=closeNawtSession&id=$_"><img src="$session{config}{extrasURL}/images/trash.gif" alt="Close session"></a></td>
				</tr>~;			
		} 
#		elsif ($item{isAdmin}) {
#			$html{admins} .= qq~<tr class=$html{rowclass}>
#				<td width=16><img src="$session{config}{extrasURL}/images/user.gif"></td>
#				<td>Guest</td>
#				<td>$item{lastIP}</td>
#				<td>$lastact</td>
#				<td>$expires</td>
#				</tr>~;				
#		}
		elsif (not $item{isAdmin} && $item{userId} =~ m/^\w{6}$/) {
			my $link;
			if ($item{userType} eq "unit") {
				$link = qq~<a href="?op=editAccount&oid=$item{userId}">$session{netams}->{units}{$item{userId}}{name} ($item{userType})</a>~;
			} elsif ($item{userType} eq "account") {
				my %acc = $session{netams}->getAccount($item{userId});
				$link = qq~<a href="?op=editAccount&oid=$item{userId}">$acc{name} ($item{userType})</a>~;
			}
			$html{users} .= qq~<tr class=$html{rowclass}>
				<td width=16><img src="$session{config}{extrasURL}/images/user.gif"></td>
				<td>$link</td>
				<td>$item{lastIP}</td>
				<td>$lastact</td>
				<td>$expires</td>
				<td><a href="?op=closeNawtSession&id=$_"><img src="$session{config}{extrasURL}/images/trash.gif" alt="Close session"></a></td>				
				</tr>~;					
		}
		$row++;
	}
	$result = qq~
	<table class=adminlist>
	<tr class=boxhead><td colspan=2>User</td><td>IP</td><td>Last action</td><td>Expires</td><td></td></tr>
	$html{admins}
	$html{users}
	</table>
	~;
	return $result;
}

#--------------------------------------------------------------------
sub getNetamsPerflog {
	my (%sSched);
		%sSched = $session{netams}->getService("scheduler");
	foreach (keys %sSched) {
		return $1 if $sSched{$_}{action} =~ m/show perf (.+)( header)?/;
	}
}

#--------------------------------------------------------------------
sub setNetamsPerflog {
	$_[0] = "/var/log/netams-perf.log" if !defined $_[0];
	$session{netams}->runCmd("show perf $_[0] header");
	$session{netams}->runCmd("service scheduler");
	$session{netams}->runCmd("time 10min action \"show perf $_[0]\"");
	$session{netams}->runCmd("exit");
	$session{netams}->runCmd("save");
}

#--------------------------------------------------------------------
sub www_createPerflog {
	checkAdmin();
	setNetamsPerflog();
	return Nawt::Theme::getRedirect("$session{env}{scripturl}?op=showInfo&fn=ChartSysload");	
}

#--------------------------------------------------------------------
sub showPerfChart {
	checkAdmin();
	my ($result, %sSched, %html, %loadDay, %loadMonth, @TIME, @LOAD, @RES, @DsAVG, @DsBps, @now);
		@now = localtime();
	# Check for perflog
	my $netamsPerfLog = getNetamsPerflog();
	if ($netamsPerfLog eq "" or not -f $netamsPerfLog) {
		return qq~ NeTAMS perflog not found! Click <a href="?op=createPerflog">here</a> to create it. <br>(It will create /var/log/netams-perf.log and add task to scheduler to update it every 10 mins.~;
	}
	
	open (FH, $netamsPerfLog);
	while (<FH>) {
		if ($_ =~ m/^\d{2}/) {
			$_ =~ s/ +/ /g;
			my @row = split " ", $_;
			push @TIME, $row[0]." ".$row[1];
			push @LOAD, $row[4];
			push @RES, $row[5];
			push @DsAVG, $row[8];
			push @DsBps, $row[11];
		}
	}
	close (FH);

	if ($session{form}{fn} =~ m/Sysload/) {
		$html{sysload} = plotGraph([\@TIME,\@LOAD],"sysload","System Load","Percents (%)");
	} elsif ($session{form}{fn} =~ m/ResUsage/) {
		$html{res} = plotGraph([\@TIME,\@RES],"sysres","Recources Usage","KBytes");
	} elsif ($session{form}{fn} =~ m/DsAvg/) {
		$html{dsavg} = plotGraph([\@TIME,\@DsAVG],"sysdsavg","Data-Source AVG","Bytes/Sec");
	} else {
		$html{dsbps} = plotGraph([\@TIME,\@DsBps],"sysdsbps","Data-Source Bps","Bytes/Sec");
	}
	
	return qq~
		$html{sysload} $html{res} $html{dsavg} $html{dsbps}
	~;
}

# ( @data , prefix , title, y_label) 
#--------------------------------------------------------------------
sub plotGraph {
	my (@keys, @values, $graph);
#		@keys = @{$_[0][0]};
#		@values = @{$_[0][1]};
#		$graph = new GD::Graph::lines( 600, 300 );
#	$graph->set( 
#		x_labels_vertical  => '1',
#		y_label           => $_[3],
#		title             => $_[2],
#		#show_values       => '1',
#		);
#	open(IMG, ">", "$session{config}{extrasDIR}/charts/$session{cookie}{nwSession}_$_[1].png") or return "Write file to $session{config}{extrasDIR}/charts/$session{cookie}{nwSession}_$_[1].png ".$!;
#	binmode IMG;
#	print IMG $graph->plot([\@keys,\@values])->png or return $!;
	return qq~<img src="$session{config}{extrasURL}/charts/$session{cookie}{nwSession}_$_[1].png">~;
}

#--------------------------------------------------------------------
sub www_showNetamsLog {
	my ($result, %html, $line, $re, @lines, $str);
	$session{page}{title} = "NeTAMS Log";
	
		$str = $session{form}{search};
		#$str =~ s/\./\\\./g;
		#$re = qr/$str/ if $session{form}{search} ne "";
	$html{filter} .= qq~
	<DIV class=databox>
	<DIV class=boxhead>Search Log For:</DIV>
	<table>
		<form name="filter" method="GET" action="$session{env}{scripturl}">
		<input type="hidden" name="op" value="showNetamsLog">
		<input type="hidden" name="fn" value="Log">
		<tr>
			<td class=bold>String:</td><td><input type="text" size=50 name="search" value="$session{form}{search}">
				<input type="submit" value="Show">
			</td>
		</tr>
		</form>
	</table>
	</DIV>
	~;
	
	# Read log
	if ($str ne "") {
		$html{log} = `cat $session{config}{nalog} | grep $str`;	
	} else {
		$html{log} = `tail -n 50 $session{config}{nalog}`;	
	}

	$result = qq~
	$html{filter}
	<p>
	<DIV class=bold>Last log entries ($session{config}{nalog}):</DIV>
	<DIV class=databox>
	<pre>
	$html{log}
	</pre>
	</DIV>

	~;
	
	return $result;
}


#--------------------------------------------------------------------
sub www_showNetamsConfig {
	my %html;
	my @lines = $session{netams}->runCmd("show Config");
	$session{page}{title} = "NeTAMS Config";
	foreach (@lines) {
		$html{body} .= "$_";
	}
	return qq~ 
	<DIV class=databox>
		<DIV class=boxhead>NeTAMS Config</DIV>
		<DIV class=boxinner>
			<pre>$html{body}</pre>
		</DIV>
	</DIV>
	~;	
}


#--------------------------------------------------------------------
sub showStatus {
	my (%html, @lines);
	$session{page}{title} = "Netams Status";
	@lines = $session{netams}->runCmd("show version");
	$html{version} .= "$_" foreach @lines;
	@lines = $session{netams}->runCmd("show processor");
	$html{proc} .= "$_" foreach @lines;
	@lines = $session{netams}->runCmd("show ds");
	$html{ds} .= "$_" foreach @lines;
	@lines = $session{netams}->runCmd("show connections");
	$html{conn} .= "$_" foreach @lines;
	#@lines = $session{netams}->runCmd("show health");
	#$html{health} .= "$_" foreach @lines;
	
	return qq~
	<DIV class=databox>
		<DIV class=boxhead>General Information</DIV>
		<DIV class=boxinner><pre>$html{version}</pre></DIV>
	</DIV>
	
	<DIV class=databox>
		<DIV class=boxhead>Service Processor Status</DIV>
		<DIV class=boxinner><pre>$html{proc}</pre></DIV>
	</DIV>
	
	<DIV class=databox>
		<DIV class=boxhead>Data Sources Status</DIV>
		<DIV class=boxinner><pre>$html{ds}</pre></DIV>
	</DIV>
	
	<DIV class=databox>
		<DIV class=boxhead>Connections</DIV>
		<DIV class=boxinner><pre>$html{conn}</pre></DIV>
	</DIV>
	~;	
}





1;