package Nawt::SQL;
# $Id: SQL.pm,v 1.1 2004/12/13 00:34:10 ramirez Exp $
use strict;
use DBI;
use Exporter;
use Tie::IxHash;
use Tie::CPHash;
use Nawt::Session;

our @ISA = qw(Exporter);
our @EXPORT = qw(&quote &getNextId );
our $COUNT = 0;

# Returns the next row of data as an array.
#-------------------------------------------------------------------
sub array {
        return $_[0]->{_sth}->fetchrow_array() or Nawt::Debug::fatalError("Couldn't fetch array. ".$_[0]->{_sth}->errstr);
}

# buildArray ( sql [, dbh ] ); @arr = Nawt::SQL->buildArray($sql);
# Builds an array of data from a series of rows.
# An SQL query. The query must select only one column of data.
#-------------------------------------------------------------------
sub buildArray {
    my ($sth, $data, @array, $i);
    $sth = Nawt::SQL->read($_[1],$_[2]);
	$i=0;
    while (($data) = $sth->array) {
        $array[$i] = $data;
		$i++;
    }
    $sth->finish;
    return @array;
}

# buildArrayRef ( sql [, dbh ] );  $arrayRef = Nawt::SQL->buildArrayRef($sql);
# Builds an array reference of data from a series of rows.
# An SQL query. The query must select only one column of data.
#-------------------------------------------------------------------
sub buildArrayRef {
	my @array = $_[0]->buildArray($_[1],$_[2]);
	return \@array;
}

# buildHash ( sql [, dbh ] ); 
# Builds a hash of data from a series of rows.
# An SQL query. The query must select at least two columns of data, the first being the key for the hash, the second being the value. 
# If the query selects more than two columns, then the last column will be the value and the remaining columns will be joined 
# together by an underscore "_" to form a complex key.
#-------------------------------------------------------------------
sub buildHash {
	my ($sth, %hash, @data);
	tie %hash, "Tie::IxHash";
        $sth = Nawt::SQL->read($_[1],$_[2]);
        while (@data = $sth->array) {
			my $value = pop @data;
			my $key = join("_",@data);	
            $hash{$key} = $value;
        }
        $sth->finish;
	return %hash;
}

# buildHashRef ( sql [, dbh ] ); $hashRef = Nawt::SQL->buildHashRef($sql);
# Builds a hash reference of data from a series of rows.
# An SQL query. The query must select at least two columns of data, the first being the key for the hash, the second being the value. 
# If the query selects more than two columns, then the last column will be the value and the remaining columns will be joined 
# together by an underscore "_" to form a complex key.
#-------------------------------------------------------------------
sub buildHashRef {
    my ($sth, %hash);
    tie %hash, "Tie::IxHash";
	%hash = $_[0]->buildHash($_[1],$_[2]);
    return \%hash;
}

# errorCode; 
# Returns an error code for the current handler.
#-------------------------------------------------------------------
sub errorCode {
    return $_[0]->{_sth}->err;
}

# errorMessage 
# Returns a text error message for the current handler.
#-------------------------------------------------------------------
sub errorMessage {
    return $_[0]->{_sth}->errstr;
}

# finish ( )
# Ends a query after calling the "new" or "read" methods.
#-------------------------------------------------------------------
sub finish {
    return $_[0]->{_sth}->finish;
}

# getColumnNames
# Returns an array of column names. Use with a "read" method.
#-------------------------------------------------------------------
sub getColumnNames {
    return @{$_[0]->{_sth}->{NAME}};
}

# getNextId ( idName )
# Increments an incrementer of the specified type and returns the value. 
# NOTE: This is not a regular method, but is an exported subroutine.
# idName: Specify the name of one of the incrementers in the incrementer table.
#-------------------------------------------------------------------
sub getNextId {
        my ($id);
        ($id) = Nawt::SQL->quickArray("select nextValue from incrementer where incrementerId='$_[0]'");
        Nawt::SQL->write("update incrementer set nextValue=nextValue+1 where incrementerId='$_[0]'");
        return $id;
}

# getRow ( table, key, keyValue [, dbh ] )
# Returns a row of data as a hash reference from the specified table.
# table: The name of the table to retrieve the row of data from.
# key: The name of the column to use as the retrieve key. Should be a primary or unique key in the table.
# keyValue: The value to search for in the key column.
#-------------------------------------------------------------------
sub getRow {
        my ($self, $table, $key, $keyValue, $dbh) = @_;
        my $row = Nawt::SQL->quickHashRef("select * from $table where ".$key."=".quote($keyValue), $dbh);
        return $row;
}

# hash ( )
# Returns the next row of data in the form of a hash. Must be executed on a statement handler returned by the "read" method.
#-------------------------------------------------------------------
sub hash {
	my ($hashRef);
        $hashRef = $_[0]->{_sth}->fetchrow_hashref();
	if (defined $hashRef) {
        	return %{$hashRef};
	} else {
		return ();
	}
}

# hashRef ( )
# Returns the next row of data in the form of a hash reference. Must be executed on a statement handler returned by the "read" method.
#-------------------------------------------------------------------
sub hashRef {
	my ($hashRef, %hash);
        $hashRef = $_[0]->{_sth}->fetchrow_hashref();
	tie %hash, 'Tie::CPHash';
    if (defined $hashRef) {
		%hash = %{$hashRef};
        return \%hash;
    } else {
        return $hashRef;
    }
}

# new ( sql [, dbh ] )
# Constructor. Returns a statement handler.
# sql: An SQL query.
#-------------------------------------------------------------------
sub new {
	my ($class, $sql, $dbh, $sth);
        $class = shift;
        $sql = shift;
        $dbh = shift || $Nawt::Session::session{dbh};
	#if ($Nawt::Session::session{setting}{showDebug}) {
	#	push(@{$Nawt::Session::session{SQLquery}},$sql);
	#}
	$COUNT++;
    $sth = $dbh->prepare($sql) or Nawt::Debug::fatalError("Couldn't prepare statement: ".$sql." : ". DBI->errstr);
    $sth->execute or Nawt::Debug::fatalError("Couldn't execute statement: ".$sql." : ". DBI->errstr);
	bless ({_sth => $sth}, $class);
}

# quickArray ( sql [, dbh ] )
# Executes a query and returns a single row of data as an array.
# sql: An SQL query.
#-------------------------------------------------------------------
sub quickArray {
	my ($sth, @data);
        $sth = Nawt::SQL->new($_[1],$_[2]);
	@data = $sth->array;
	$sth->finish;
	return @data;
}

# quickCSV ( sql [, dbh ] )
# Executes a query and returns a comma delimited text blob with column headers.
# sql: An SQL query.
#-------------------------------------------------------------------
sub quickCSV {
        my ($sth, $output, @data);
        $sth = Nawt::SQL->new($_[1],$_[2]);
        $output = join(",",$sth->getColumnNames)."\n";
        while (@data = $sth->array) {
                makeArrayCommaSafe(\@data);
                $output .= join(",",@data)."\n";
        }
        $sth->finish;
        return $output;
}

# quickHash ( sql [, dbh ] )
# Executes a query and returns a single row of data as a hash.
# sql: An SQL query.
#-------------------------------------------------------------------
sub quickHash {
    my ($sth, $data);
        $sth = Nawt::SQL->new($_[1],$_[2]);
        $data = $sth->hashRef;
        $sth->finish;
	if (defined $data) {
        return %{$data};
	} else {
		return ();
	}
}

# quickHashRef ( sql [, dbh ] )
# Executes a query and returns a single row of data as a hash reference.
# sql: An SQL query.
#-------------------------------------------------------------------
sub quickHashRef {
        my ($sth, $data);
        $sth = Nawt::SQL->new($_[1],$_[2]);
        $data = $sth->hashRef;
        $sth->finish;
        if (defined $data) {
                return $data;
        } else {
		return {};
	}
}

# quickTab ( sql [, dbh ] )
# Executes a query and returns a tab delimited text blob with column headers.
# sql: An SQL query.
#-------------------------------------------------------------------
sub quickTab {
    my ($sth, $output, @data);
    $sth = Nawt::SQL->new($_[1],$_[2]);
	$output = join("\t",$sth->getColumnNames)."\n";
	while (@data = $sth->array) {
        makeArrayTabSafe(\@data);
    	$output .= join("\t",@data)."\n";
    }
    $sth->finish;
	return $output;
}

#-------------------------------------------------------------------
sub makeArrayTabSafe {
	my $array = $_[0];
	$_ = makeTabSafe($_) for @$array;
}

#-------------------------------------------------------------------
sub makeTabSafe {
	my $text = $_[0];
	$text =~ tr/\r\n/ /;
	$text =~ s/\t/    /g;
	return $text;
}

# quote ( string ) 
# Returns a string quoted and ready for insert into the database.  
# NOTE: This is not a regular method, but is an exported subroutine.
# string: Any scalar variable that needs to be escaped to be inserted into the database.
#-------------------------------------------------------------------
sub quote {
    my $value = $_[0]; #had to add this here cuz Tie::CPHash variables cause problems otherwise.
    return $Nawt::Session::session{dbh}->quote($value);
}

# read ( sql [, dbh ] )
# An alias of the "new" method. Returns a statement handler.
# sql: An SQL query.
#-------------------------------------------------------------------
sub read {
  	return Nawt::SQL->new($_[1],$_[2],$_[3]);
}

# rows ( )
# Returns the number of rows in a statement handler created by the "read" method.
#-------------------------------------------------------------------
sub rows {
    return $_[0]->{_sth}->rows;
}

# setRow ( table, key, data [, dbh ] )
# Inserts/updates a row of data into the database. Returns the value of the key.
#  table: The name of the table to use.
#  key: The name of the primary key of the table.
#  data: A hash reference containing column names and values to be set.
#-------------------------------------------------------------------
sub setRow {
        my ($self, $table, $keyColumn, $data, $dbh) = @_;
        if ($data->{$keyColumn} eq "new") {
                $data->{$keyColumn} = getNextId($keyColumn);
                Nawt::SQL->write("insert into $table ($keyColumn) values ($data->{$keyColumn})", $dbh);
        }
        my (@pairs);
        foreach my $key (keys %{$data}) {
                unless ($key eq $keyColumn) {
                        push(@pairs, $key.'='.quote($data->{$key}));
                }
        }
        Nawt::SQL->write("update $table set ".join(", ", @pairs)." where ".$keyColumn."=".quote($data->{$keyColumn}), $dbh);
	return $data->{$keyColumn};
}

# unconditionalRead ( sql [, dbh ] )
# An alias of the "read" method except that it will not cause a fatal error in Nawt if the query is invalid. 
# This is useful for user generated queries such as those in the SQL Report. Returns a statement handler.
#  sql: An SQL query.
#-------------------------------------------------------------------
sub unconditionalRead {
        my ($sth,$dbh);
	$dbh = $_[2] || $Nawt::Session::session{dbh};
        $sth = $dbh->prepare($_[1]) or Nawt::Debug::warn("Unconditional read failed: ".$_[1]." : ".DBI->errstr);
        if ($sth) {
        	$sth->execute or Nawt::Debug::warn("Unconditional read failed: ".$_[1]." : ".DBI->errstr);
        	bless ({_sth => $sth}, $_[0]);
        }       
}

# write ( sql [, dbh ] )
# A method specifically designed for writing to the database in an efficient manner. 
# Writing can be accomplished using the "new" method, but it is not as efficient.
#  sql: An SQL insert or update.
#-------------------------------------------------------------------
sub write {
	my ($dbh);
	$dbh = $_[2] || $Nawt::Session::session{dbh};
    $dbh->do($_[1]) or Nawt::Debug::fatalError("Couldn't prepare statement: ".$_[1]." : ". DBI->errstr);
}

1;