package Nawt::Session;
# $Id: Session.pm,v 1.8 2005/04/20 13:01:12 ramirez Exp $

use DBI;
use strict;
use Netams;
use Exporter; 
use CGI;
use Parse::PlainConfig;
use Nawt::Debug;
use Nawt::SQL;
use Nawt::Language;

our @ISA = qw( Exporter ); 
our @EXPORT = qw( %session &round &bytesToHuman);
our %session = ();
#tie %session, "Tie::IxHash";

#    
# :    root path, , mode
# :      no
#-------------------------------------------------------------------
sub open {
	my ($key, $config);
	$session{user}{isAdmin} = $_[2];
	$session{options}{mode} = $_[2];
	###----------------------------
	### operating system specific things
	$session{os}{name} = $^O;
	if ($session{os}{name} =~ /^Win/i) {
		$session{os}{type} = "Windows";
		$session{os}{slash} = "\\";
	} else {
		$session{os}{type} = "Unix";
		$session{os}{slash} = "/";
	}
	###----------------------------
	### config file variables
	$session{config}{nawtRoot} = $_[0] || "/var/www/apache2/addon-modules/nawt";
	$session{config}{nawtRoot} =~ s/\/$//g;
	$session{config}{configFile} = $_[1] || "nawt.conf";
	$config = Parse::PlainConfig->new('DELIM' => '=', 'FILE' => $session{config}{nawtRoot}.'/etc/'.$session{config}{configFile}, 'PURGE' => 1);
    foreach ($config->directives) {	$session{config}{$_} = $config->get($_); }
	   
	#if( defined( $session{config}{scripturl} ) ) {
	#	# get rid of leading "/" if present.
	#	$session{config}{scripturl} =~ s/^\///;
	#} else {
		# default to the "real" path to script.
		$session{config}{scripturl} = $ENV{SCRIPT_NAME};
	#}
	# remove trailing slash
	$session{config}{extrasURL} =~ s/\/$//g;
	###----------------------------
	### creating SQL database handler object
	$session{dbh} = DBI->connect($session{config}{dsn},$session{config}{dbuser},$session{config}{dbpass},{ RaiseError=>0,AutoCommit=>1 });
	if ( $session{config}{dsn} =~ /Oracle/ ) { # Set Oracle specific attributes
		$session{dbh}->{LongReadLen} = 512 * 1024;
		$session{dbh}->{LongTruncOk} = 1;
	}
	###----------------------------
	### creating CGI object
	$CGI::POST_MAX=1024 * 1000;
	$session{cgi} = CGI->new();
    if ($session{cgi}->cgi_error =~ /^413/) {
		$session{header}{status} = $session{cgi}->cgi_error;
		$CGI::POST_MAX=-1;
		$session{cgi} = CGI->new();
    }
	###----------------------------
	### header variables
	$session{header}{mimetype} = 'text/html';
	###----------------------------
	### evironment variables from web server
	$session{env} = \%ENV;
	### check to see if client is proxied and adjust remote_addr as necessary
	if (($session{config}{proxiedClientAddress} eq "1") && ($ENV{HTTP_X_FORWARDED_FOR} ne "")) {
		$session{env}{REMOTE_ADDR} = $ENV{HTTP_X_FORWARDED_FOR};
	}
	###----------------------------
	### form variables
	foreach ($session{cgi}->param) {
		$session{form}{$_} = $session{cgi}->param($_);
	}
	###----------------------------
	### cookies
	foreach ($session{cgi}->cookie) {
		$session{cookie}{$_} = $session{cgi}->cookie($_);
	}
	###----------------------------
	### session variables 
	_closeOldSessions();
	if ($session{user}{isAdmin}) {
		if ($session{cookie}{NetamsAdmintoolSession} eq "") {
			start(-1); #setting up a visitor session
		} else {
			_setupSessionVars($session{cookie}{NetamsAdmintoolSession});
		}
	} else {
		if ($session{cookie}{NetamsUsertoolSession} eq "") {
			start(-1); #setting up a visitor session
		} else {
			_setupSessionVars($session{cookie}{NetamsUsertoolSession});
		}	
	}
	###----------------------------
	### Loading Language 
	if (defined $session{config}{language}) {
		eval "use Nawt::Language::$session{config}{language}";
	}
}

# ( user , pass ) 
#-------------------------------------------------------------------
sub makeNetamsConn {
	my ($user, $pass);
	if ($session{user}{isAdmin}) {
		$user = $_[0];
		$pass = $_[1];
	} else {
		$user = $session{config}{nauser};
		$pass = $session{config}{napass};
	}
	$session{netams} = Netams->new(host => $session{config}{nahost}, 
								   port => $session{config}{naport}, 
								   user => $user, 
								   pass => $pass);
	return $session{netams}->connect;
}

#-------------------------------------------------------------------
sub _setupSessionVars {
	my (%vars, $uid, $encryptedPassword);
	if ($_[0] ne "") {
		%vars = Nawt::SQL->quickHash("select * from nawt_session where sessionId='$_[0]'");
		$session{var} = \%vars;
		if ($vars{sessionId} ne "") {
			if (($session{config}{proxiedClientAddress} eq "1") && ($ENV{HTTP_X_FORWARDED_FOR} ne "")) {
				Nawt::SQL->write("update nawt_session set lastPageView="._time().", lastIP='$ENV{HTTP_X_FORWARDED_FOR}', expires=".(_time()+$session{config}{sessionTimeout})." where sessionId='$_[0]'");
			} else {
				Nawt::SQL->write("update nawt_session set lastPageView="._time().", lastIP='$ENV{REMOTE_ADDR}', expires=".(_time()+$session{config}{sessionTimeout})." where sessionId='$_[0]'");
			}
		} else {
			start(-1,$_[0]);
		}
	}

}

# convertVisitorToUser ( sessionId , userId , isAdmin , cuser , cpass [ , userType ] )
#-------------------------------------------------------------------
sub convertVisitorToUser {
	#Nawt::ErrorHandler::writeLog("AUDIT - Login. User 'Visitor' -> ");
	my $userType;
	$userType = ",userType=".quote($_[5]) if $_[5] ne "";
	Nawt::SQL->write("UPDATE nawt_session SET userId='$_[1]', cUser='$_[3]', cPass='$_[4]' $userType WHERE sessionId=".quote($_[0]));
	$session{var}{userId} = $_[1];
	#refreshUserInfo($_[1]);
}

# closes sesssion
#-------------------------------------------------------------------
sub close {
	$session{dbh}->disconnect();
	$session{netams}->close();
	undef %session;
}

#-------------------------------------------------------------------
# This routine returns an unique session Id.
sub _uniqueSessionId {
	my $sessionId = crypt((_time()*rand(1000)),rand(99));
		$sessionId =~ s/\W/0/g;
	my ($isDuplicate) = Nawt::SQL->buildArray("select count(*) from nawt_session where sessionId =".quote($sessionId));
	if ($isDuplicate) {
		return _uniqueSessionId();
	} else {
		return $sessionId;
	}
}

#-------------------------------------------------------------------
# Creates new session id in db
sub start {
	my ($sessionId,$mode,$ip);
		$mode = $session{user}{isAdmin} || 0;
		$sessionId = $_[1] || _uniqueSessionId();
	if (($session{config}{proxiedClientAddress} eq "1") && ($ENV{HTTP_X_FORWARDED_FOR} ne "")) {
		$ip = $ENV{HTTP_X_FORWARDED_FOR};
	} else {
		$ip = $ENV{REMOTE_ADDR};
	}
	Nawt::SQL->write("INSERT INTO nawt_session SET 
		sessionId='$sessionId', 
		expires=".(_time()+$session{config}{sessionTimeout}).", 
		lastPageView="._time().", 
		lastIP='$ip', 
		userId=$_[0], 
		isAdmin=$mode");	
	if ($session{user}{isAdmin}) {
		setCookie("NetamsAdmintoolSession",$sessionId);
	} else {
		setCookie("NetamsUsertoolSession",$sessionId);
	}
	refreshSessionVars($sessionId);
}

#-------------------------------------------------------------------
sub _closeOldSessions {
	Nawt::SQL->write("delete from nawt_session where expires < "._time());	
}

#-------------------------------------------------------------------
sub refreshSessionVars {
	_setupSessionVars($_[0]);
	#refreshUserInfo($session{var}{userId});
}

#-------------------------------------------------------------------
# Removes session id from db
sub end {
	Nawt::SQL->write("delete from nawt_session where sessionId='$_[0]'",$session{dbh});
	if ($_[0] eq $session{var}{sessionId}) {
		delete $session{user};
		delete $session{var};
	}
}

#-------------------------------------------------------------------
sub setCookie {
    my $ttl = $_[2] || '+10y';
    my $domain = $session{env}{SERVER_NAME} if ($session{env}{HTTP_USER_AGENT} =~ m/MSIE/i);
    push @{$session{header}{cookie}}, $session{cgi}->cookie(
		-name => $_[0],
		-value => $_[1],
		-expires => $ttl,
		# TODO: need to deal with path variable
		-path => '/', 
		-domain => $domain
		);
}

#-------------------------------------------------------------------
sub httpHeader {
	unless ($session{header}{charset}) {
		$session{header}{charset} = $LANG{CHARSET};
	}
	if ($session{header}{filename} && $session{header}{mimetype} eq "text/html") {
		$session{header}{mimetype} = "application/octet-stream";
	}
	return $session{cgi}->header( 
		-type => $session{header}{mimetype}.'; charset='.$session{header}{charset},
		-cookie => $session{header}{cookie}, 
		-status => $session{header}{status},
		-attachment => $session{header}{filename}
		);
}

#-------------------------------------------------------------------
sub _time {
	return time;
}

#-------------------------------------------------------------------
sub httpRedirect {
	return $session{cgi}->redirect($_[0]);
}

#-------------------------------------------------------------------
sub getRedirect {
	my $result=qq~ <p>
		<DIV align="center" valign="middle" width="95%" height="95%">
			Processing operation, please wait...
		</DIV>
	<script>
		window.onload = redirect();
		function redirect() {
			document.location.href="$_[0]";
		}
	</script> ~;
	return $result;
}

#-------------------------------------------------------------------
sub round {
	my $significantDigits = $_[1] || 0;
        return sprintf(('%.'.$significantDigits.'f'), $_[0]);
}

# $str = bytesToHuman( indeger )
#--------------------------------------------------------------------
sub bytesToHuman {
	my ($int, $result, $prefix, $kb);
		$int = $_[0];
		$kb = $session{config}{kilobyte};
	# determing prefix
	if ($int !~ m/^\d+$/) {
		return "";
	} elsif ($int >= 1000000000000) {
		return sprintf("%3.3f",$int/($kb*$kb*$kb*$kb))." T";
	} elsif ($int >= 1000000000) {
		return sprintf("%3.3f",$int/($kb*$kb*$kb))." G";
	} elsif ($int >= 1000000) {
		return sprintf("%3.3f",$int/($kb*$kb))." M";
	} elsif ($int >= 1000) {
		return sprintf("%3.3f",$int/$kb)." K";
	} else {
		return $int." B";
	}

}

1;