package Netams;
# $Id: Netams.pm,v 1.30 2005/04/20 14:28:42 ramirez Exp $
our $VERSION = "1.5.6";

use 5.8.0;
use strict;
use Net::Telnet;

#-------------------------------------------------------------------
sub new {
	my ($class, $self, $options, $config, $res, $ver, $t);
		$class = shift;
		$self = {};
		$options = {};
		$config = {};
	
		bless $self, $class;
		$self->{options} = $options;

	# setting default options
	%$options = (host => 'localhost', 
				 port => 20001,
				 user => 'admin',
				 pass => '123' );

	# overriding defaults with in-line options
	for (my $i = 0; $i <= $#_; $i += 2) {
		defined($_[($i + 1)]) or die ("Netams->new() called with odd number of option parameters - should be of the form option => value");
		$options->{lc($_[$i])} = $_[($i + 1)]; 
	}

	# creating connection handler object
	$self->{nth} = new Net::Telnet (Port => $options->{port},
					Host => $options->{host},
					Timeout => 5,
					Telnetmode => 0,
					Prompt => '/.*(>|#) *$/',
					#Dump_Log => '/tmp/dumplog.txt'
					);
	return $self;
}

# establishing connection, and trying to login
#-------------------------------------------------------------------
sub connect {
	my ($self, @lines, $out);
		$self = shift;
	$self->{nth}->open();
	$self->{nth}->getline;
	$out = $self->{nth}->getline;
	return 400 unless $out =~ /NeTAMS version (\d.\d.\d) \(build ([\d\.]+)\)/; # e.g. this is not netams
	$self->{options}{version} = "$1 ($2)"; # Store netams version
	return 401 unless $self->{options}{user} || $self->{options}{pass};
	@lines = $self->{nth}->waitfor('/login >$/'); # Witing for login promt
	$self->{nth}->print($self->{options}{user}); # Send login
	@lines = $self->{nth}->waitfor('/(login|password) >$/'); # Wait for answer
	return 401 unless $lines[1] =~ /password >$/; # e.g. bad login
	$self->{nth}->print($self->{options}{pass}); # Send password
	@lines = $self->{nth}->waitfor('/(login|Welcome).*/i'); # Wait for answer
	return 402 unless $lines[1] =~ /Welcome/i; # e.g. bad login
	$self->{options}{welcome} = $lines[1];
	$self->{nth}->waitfor('/>$/'); # Wait for promt
	return 200; # e.g. connected
}

# reading and parsing config
#-------------------------------------------------------------------
#sub loadData {
#	my ($self, $config);
#		$self = shift;
#	%$config = _parseConfig($self);
#	$self->{config} = $config;
#	return 1;
#}

#-------------------------------------------------------------------
sub getUserOidByName {
	my $self = shift;
	foreach (keys %{$self->{config}{services}{main}{users}}) {
		return $_ if $self->{config}{services}{main}{users}{$_}{name} =~ /$_[0]/i;
	}
}

# hasUnit ( oid )
#-------------------------------------------------------------------
sub hasUnit {
	my $self = shift;
	return 1 if $_[0] ne "" and exists $self->{config}{units}{$_[0]};
	return 0;
}

# hasUser ( oid )
# Returns 1 if User with OID exists
#-------------------------------------------------------------------
sub hasUser {
	my $self = shift;
	return 1 if $_[0] ne "" and exists $self->{config}{services}{main}{users}{$_[0]};
	return 0;
}

# hasPolicy ( oid )
#-------------------------------------------------------------------
sub hasPolicy {
	my ($self);
		$self = shift;
	return 1 if $_[0] ne "" and exists $self->{config}{policies}{$_[0]};
	return 0;
}

# boolean = hasService ( name )
#-------------------------------------------------------------------
sub hasService {
	my ($self, $result);
		$self = shift;
	foreach (keys  %{$self->{config}{services}}) {
		return 1 if $_ =~ m/$_[0]/;
	}	
}

# %hash = getService ( [ name ] , [ mode ] )
#-------------------------------------------------------------------
sub getService {
	my ($self, $result);
		$self = shift;
	if (!defined $_[0]) {
		return %{$self->{config}{services}};
	} elsif (!defined $_[1]) {
		foreach (keys %{$self->{config}{services}}) {
			return %{$self->{config}{services}{$_}} if $_ =~ m/$_[0]/;
		}
	} elsif (defined $_[0] && defined $_[1]) {
		my %data = %{$self->{config}{services}};
		foreach (keys %data) {
			delete $data{$_} if $_ !~ m/$_[0]/;
		}
		return %data;
	}
}

# accountOid = unitInBilling ( [ oid ] )
#-------------------------------------------------------------------
sub unitInBilling {
	my ($self, $unitId, $accId);
		$self = shift;
		$unitId = shift;
	foreach (keys %{$self->{config}{accounts}}) {
		if ($self->{config}{accounts}{$_}{units} =~ m/$unitId/) {
			return $_;
		}
	}
}

# hash = getAccount( [ oid ] )
#-------------------------------------------------------------------
sub getAccount {
	my ($self, %bill, @lines, $accId, $oid);
		$self = shift;
		$oid = shift;
	if ($oid ne "") {
		return %{$self->{config}{accounts}{$oid}} if defined $self->{config}{accounts}{$oid};
	}
	return %{$self->{config}{accounts}} if defined $self->{config}{accounts};
}

# boolean = hasAccount ( oid )
#-------------------------------------------------------------------
sub hasAccount {
	my ($self, %list);	
	$self = shift;
	return 1 if $self->hasService('billing') and defined $self->{config}{accounts}{$_[0]};
	return 0;
}


#-------------------------------------------------------------------
sub getPlan {
	my ($self, %data, @lines);
		$self = shift;
		%data = $self->getService("billing");
	return 0;
}


# Parses output of "show login" command
#-------------------------------------------------------------------
sub getLogin {
	my ($self, %data, @lines, %re);
		$self = shift;
		@lines = $self->{nth}->cmd("show login");
		$re{oid} = qr/OID: (\w{6}) \((\w+)\) (CLOSED|OPENED), inact:(\d+), abs:(\d+)/;
		$re{op} = qr/OP:([\d{2}\.]+ [\d{2}:]+), LU:([\d\.]+ [\d{2}:]+), ([\d{1,3}\.]+)/;
	my $oid;
	foreach (@lines) {
		if ($_ =~ m/$re{oid}/i) {
			$oid = $1;
			$data{$oid}{status} = $3;
			$data{$oid}{inact} = $4;
			$data{$oid}{abs} = $5;
		} elsif ($_ =~ m/$re{op}/i) {
			$data{$oid}{opened} = $1;
			$data{$oid}{used} = $2;
			$data{$oid}{ip} = $3;
		}
	}
	return %data;
}

# parses ouput "show monitir" command
#-------------------------------------------------------------------
sub getMonitor {
	my ($self, %data, @lines);
		$self = shift;
		@lines = $self->{nth}->cmd("show monitor");
	my $oid;
	my $mon;
	foreach (@lines) {
		$mon = $1 if $_ =~ m/service (monitor \d+)/;
		if ($_ =~ m/Units: (.+)/) {
			my $str; $str = $1;
			foreach (split(" ",$str)) {
				if ($_ =~ m/[\w-]+\((\w{6})\)/) {
					$data{$1} .= "$mon,";
				}
			}
		}
			
	}
	
	return %data;
}

# getQuota ( [ oid ] )
# returns hash with quota values for all units: $hash{oid}{param}=value
# if defined oid returns values only for this unit: $hash{param}=value
#-------------------------------------------------------------------
sub getQuota {
	my ($self, %data, @lines);
		$self = shift;
		
	if (defined $_[0]) {
		@lines = $self->{nth}->cmd("show quota oid $_[0]");
	} else {
		@lines = $self->{nth}->cmd("show quota");
	}

	my $oid;
	foreach (@lines) {
		$_ =~ s/\s/ /g;
		$_ =~ s/ +/ /g;
		if ($_ =~ m/OID: (\w{6}) \([\w-]+\) policy: ([\w-]+) soft-treshold: (\d+)% (ACTIVE)?/) {
			$oid = $1;
			$data{$oid}{policy} = $2;
			$data{$oid}{treshold} = $3;
			if ($4 ne "") {
				$data{$oid}{active} = 1;
			} else {
				$data{$oid}{active} = 0;
			}
		} elsif ($_ =~ m/(HOUR|DAY|WEEK|MONTH) +(in|out): (\d+), (soft|hard)quota (\d+\.\d+\w) ratio (\d+)%/) {
			if ($oid ne "") {
				$data{$oid}{$1}{$2}{$4}{used} = $3;
				$data{$oid}{$1}{$2}{$4}{limit} = $5;
				$data{$oid}{$1}{$2}{$4}{ratio} = $6;
			}
		}
	}
	
	return %data;
}

# boolean = checkVersion ( version , build ) 
# check for netams version
#-------------------------------------------------------------------
sub checkVersion {
	my ($self, $ver);
		$self = shift;
		$ver = $self->{config}{version};
	$ver =~ m/(\d\.\d\.\d) \(build (\d+)\)/g;
	return 1 if $1 eq $_[0] && $2 >= $_[1];
	return 0;
}

# returns NeTAMS version
#-------------------------------------------------------------------
sub getVersion {
	my ($self);
		$self = shift;
	return $self->{config}{version};
}

# usage = getGroupChilds( [ name ] )
#-------------------------------------------------------------------
sub getGroupChilds {
	my ($self, $oid);
		$self = shift;
	if (defined $self->{config}{childs}{$_[0]}) { 
		return @{$self->{config}{childs}};
	} else {
		return %{$self->{config}{childs}};
	}

}

# getUnit ( [ oid ] )
# returns hash with unit params
#-------------------------------------------------------------------
sub getUnit {
	my ($self, $result, @lines);
		$self = shift;
	if (!defined $_[0]) {
		return %{$self->{config}{units}} if defined $self->{config}{units};
	} elsif ($_[0] eq "groups") {
		return %{$self->{config}{groups}} if defined $self->{config}{groups};
	} else {
		return %{$self->{config}{units}{$_[0]}} if defined $self->{config}{units}{$_[0]};
	}
}

# usage = getUser( oid )
#-------------------------------------------------------------------
sub getUser {
	my ($self, $result, @lines);
		$self = shift;
	if (!defined $_[0]) {
		return %{$self->{config}{services}{main}{users}};
	} else {
		return %{$self->{config}{services}{main}{users}{$_[0]}};
	}
}

# usage = remUnit( oid )
#-------------------------------------------------------------------
sub remUnit {
	my ($self, %unit);
		$self = shift;
	if ($self->hasUnit($_[0])) {
		%unit = $self->getUnit($_[0]);
		$self->runCmd("service processor");
		$self->runCmd("no unit $unit{type} oid $_[0]");
		$self->runCmd("exit");
		$self->runCmd("save");
	}

}

# setUser ( oid => oid , param => value ... )
# stores params in NeTAMS. Returns OID.
#-------------------------------------------------------------------
sub setUser {
	my ($self, %ptrs, %user, $cmd, @out, $oid);
		$self = shift;
		%ptrs = (@_);
	if ($ptrs{oid} ne "" and $self->hasUser($ptrs{oid})) {
		%user = $self->getUser($ptrs{oid});
		$oid = $ptrs{oid};
	} else {
		$ptrs{name} = "NewUser" if $ptrs{name} !~ m/^\S+$/;
		$ptrs{password} = "123" if $ptrs{password} !~ m/^\S+$/;
		@out = $self->{nth}->cmd("user name $ptrs{name} password $ptrs{password}");
		$out[0] =~ m/user [\w-_]+ (\w{6}) created/;
		$ptrs{oid} = $1;
		$oid = $1;
	}
	# NAME
	if (defined $ptrs{name} && $ptrs{name} ne "" && $ptrs{name} ne $user{name}) {
		$self->runCmd("user oid $ptrs{oid} name $ptrs{name}");
	}
	# REAL-NAME
	if (defined $ptrs{'real-name'} && $ptrs{'real-name'} ne "" && $ptrs{'real-name'} ne $user{'real-name'}) {
		$self->runCmd("user oid $ptrs{oid} real-name \"$ptrs{'real-name'}\"");
	}
	# EMAIL
	if (defined $ptrs{email} && $ptrs{email} ne "" && $ptrs{email} ne $user{email}) {
		$self->runCmd("user oid $ptrs{oid} email $ptrs{email}");
	} elsif ($ptrs{email} eq "" && $user{email} ne "") {
		$self->runCmd("user oid $ptrs{oid} email \"\"");
	}
	# PASSWORD
	if (defined $ptrs{password} && $ptrs{password} ne "" && $ptrs{password} ne $user{password}) {
		$self->runCmd("user oid $ptrs{oid} password $ptrs{password}");
	}
	# PERMIT
	if (defined $ptrs{permit} && $ptrs{permit} ne "" && $ptrs{permit} ne $user{permit}) {
		$self->runCmd("user oid $ptrs{oid} permit $ptrs{permit}");
	}
	return $oid;
}


#-------------------------------------------------------------------
sub setAccount {
	my ($self, %ptrs, %item, $cmd, @out);
		$self = shift;
		%ptrs = (@_);
		%item = $self->getAccount($ptrs{oid});
	if ($self->hasService('billing')) {
		$self->runCmd("service billing");
		# Check if OID exist
		if (not $self->hasAccount($ptrs{oid})) {
			$ptrs{name} = "NewAccount" if $ptrs{name} eq "";
			$ptrs{password} = "123" if $ptrs{password} eq "";
			@out = $self->runCmd("account $ptrs{name} password $ptrs{password}");
			$out[0] =~ m/account (\w{6}) created/;
			$ptrs{oid} = $1;
		}
		# NAME
		if ($ptrs{name} ne "" && $ptrs{name} ne $item{name}) {
			$self->runCmd("account $ptrs{oid} name \"$ptrs{name}\"");
		}
		# DESCRIPTION
		if ($ptrs{description} ne "" && $ptrs{description} ne $item{description}) {
			$self->runCmd("account $ptrs{oid} description \"$ptrs{description}\"");
		}
		# PASSWORD
		if ($ptrs{password} ne "" && $ptrs{password} ne $item{password}) {
			$self->runCmd("account $ptrs{oid} password $ptrs{password}");
		}		
		# BALANCE
#		if ($ptrs{balance} ne "" && $ptrs{balance} != $item{balance}) {
#			$self->runCmd("account $ptrs{oid} balance set $ptrs{balance}");
#		}
		# CREDIT
		if ($ptrs{credit} ne "" && $ptrs{credit} != $item{credit}) {
			$self->runCmd("account $ptrs{oid} credit-limit $ptrs{credit}");
		}
		# BLOCK | UNBLOCK
		if ($ptrs{block}) {
			$self->runCmd("account $ptrs{oid} block");
		} else {
			$self->runCmd("account $ptrs{oid} unblock");
		}
		# PLAN
		if ($ptrs{plan} ne "" && $ptrs{plan} ne $item{plan}) {
			$self->runCmd("account $ptrs{oid} plan $ptrs{plan}");
		}			
		$self->runCmd("exit");
		$self->runCmd("save");
	}
	return $ptrs{oid};
}

# setUnit ( oid => oid , type => type , param => value ... )
# stores params in NeTAMS. Returns OID.
#-------------------------------------------------------------------
sub setUnit {
	my ($self, %ptrs, %unit, $cmd, @out);
		$self = shift;
		%ptrs = (@_);		
		
	$self->runCmd("service processor");
	# Check if OID exist
	if ($ptrs{oid} ne "" && $self->hasUnit($ptrs{oid})) {
		%unit = $self->getUnit($ptrs{oid});
	} else { 
		$ptrs{name} = "_unnamed_" if $ptrs{name} !~ m/^\S+$/;
		$ptrs{type} = "group" if !defined $ptrs{type};
		$unit{type} = $ptrs{type};
		@out = $self->{nth}->cmd("unit $ptrs{type} name $ptrs{name}");
		$out[0] =~ m/^parse: unit $ptrs{type} (\w{6})/;
		$ptrs{oid} = $1;
	}
	
	# NAME
	if (defined $ptrs{name} && $ptrs{name} ne "" && $ptrs{name} ne $unit{name}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} name $ptrs{name}");
	}
	# PARENT
	if (defined $ptrs{parent} && $ptrs{parent} ne "" && $ptrs{parent} ne $unit{parent} && $ptrs{parent} ne $unit{name}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} parent $ptrs{parent}");
	} elsif (defined $ptrs{parent} && $ptrs{parent} eq "" && $ptrs{parent} ne $unit{parent}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no parent");
	}
	# PASSWORD
	if (defined $ptrs{password} && $ptrs{password} ne "" && $ptrs{password} ne $unit{password}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} password $ptrs{password}");
	} elsif (defined $ptrs{password} && $ptrs{password} eq "" && $ptrs{password} ne $unit{password}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no password");
	}
	# EMAIL
	if (defined $ptrs{email} && $ptrs{email} ne "" && $ptrs{email} ne $unit{email}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} email $ptrs{email}");
	} elsif (defined $ptrs{email} && $ptrs{email} eq "" && $ptrs{email} ne $unit{email}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no email");
	}
	# DESCRIPTION
	if (defined $ptrs{description} && $ptrs{description} ne "" && $ptrs{description} ne $unit{description}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} description \"$ptrs{description}\"");
	} elsif (defined $ptrs{description} && $ptrs{description} eq "" && $ptrs{description} ne $unit{description}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no description");
	}	
	# NO-LOCAL-PASS
	if (defined $ptrs{nolp} && $ptrs{nolp} == 1 && $ptrs{nolp} ne $unit{nolp}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no-local-pass");
	} elsif (!defined $ptrs{nolp} && $ptrs{nolp} == 0 && $ptrs{nolp} ne $unit{nolp}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no no-local-pass");
	}
	# DS-LIST
	if (defined $ptrs{'ds-list'} && $ptrs{'ds-list'} ne "" && $ptrs{'ds-list'} ne $unit{'ds-list'}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no ds-list $_") foreach (split(",",$unit{'ds-list'}));
		$self->runCmd("unit $unit{type} oid $ptrs{oid} ds-list $ptrs{'ds-list'}");
	} elsif (!defined $ptrs{'ds-list'} && $ptrs{'ds-list'} eq "" && $ptrs{'ds-list'} ne $unit{'ds-list'}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no ds-list $unit{'ds-list'}");
	}	
	# IP ADRESS
	if (defined $ptrs{ip} && $ptrs{ip} ne "" && $unit{type} =~ m/host|user|net/ && $ptrs{ip} ne $unit{ip}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} ip $ptrs{ip}");
	} elsif (defined $ptrs{ip} && $ptrs{ip} eq "" && $unit{type} =~ m/host|user|net/) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no ip");
	}
	# IP MASK
	if (defined $ptrs{mask} && $ptrs{mask} ne "" && $unit{type} =~ /net/) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} mask $ptrs{mask}");
	}
	# IP LIST
	if ($unit{type} eq "cluster") {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no ip $_") foreach split " ",$unit{ip};
		$self->runCmd("unit $unit{type} oid $ptrs{oid} ip $_") foreach split " ",$ptrs{ip};
	}
	# MAC ADRESS
	if (defined $ptrs{mac} && $ptrs{mac} ne "" && $unit{type} =~ m/host|user/ && $ptrs{mac} ne $unit{mac}) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} mac $ptrs{mac}");
	} elsif (defined $ptrs{mac} && $ptrs{mac} eq "" && $unit{type} =~ m/host|user/) {
		$self->runCmd("unit $unit{type} oid $ptrs{oid} no mac");
	}
		
	$self->runCmd("save");
	return $ptrs{oid};
}

# setUnitPolicy ( type=>[acct|fw], uid=>oid, pid=>pid, pnum=>pnum, params ...)
#-------------------------------------------------------------------
sub setUnitPolicy {
	my ($self, %ptrs);
		$self = shift;
		%ptrs = (@_);
	
	$self->runCmd("service processor");

	# TODO: How make it?
	
	$self->runCmd("save");	
}

# getPolicy ( [ oid ] )
# returns hash with policy params
#-------------------------------------------------------------------
sub getPolicy {
	my ($self);
		$self = shift;
	if (!defined $_[0]) {
		return %{$self->{config}{policies}};
	} else {
		return %{$self->{config}{policies}{$_[0]}};
	}
}

# setBalance ( oid=>oid, sum=>value, mode=>[add|set|remove] )
#-------------------------------------------------------------------
sub setBalance {
	my ($self, %ptrs, %policy, $cmd);
		$self = shift;
		%ptrs = (@_);
	if ($self->hasService("billing") && $self->hasAccount($ptrs{oid})) {
		$self->runCmd("service billing");
		$self->runCmd("account $ptrs{oid} balance $ptrs{mode} $ptrs{sum}");
		$self->runCmd("exit");
		$self->runCmd("save");
	}
	return $ptrs{oid};
}

# setPolicy ( oid=>oid, param=>value ... )
#-------------------------------------------------------------------
sub setPolicy {
	my ($self, %ptrs, %policy, $cmd);
		$self = shift;
		%ptrs = (@_);
		
 	$self->runCmd("service processor");
 	# Check if OID exist
 	if ($ptrs{oid} ne "" && $self->hasPolicy($ptrs{oid})) {
 		%policy = $self->getPolicy($ptrs{oid});
 	} else { 
 		$ptrs{name} = "_unnamed_" if $ptrs{name} !~ m/^[\w-_]+$/;
 		my @out = $self->{nth}->cmd("policy name $ptrs{name}");
 		$out[0] =~ m/^parse: policy (\w{6}) created/;
 		$ptrs{oid} = $1;
 	}
 	$cmd = "policy oid $ptrs{oid}";
 	# NAME
 	if (defined $ptrs{name} && $ptrs{name} ne "" && $ptrs{name} ne $policy{name}) {
 		$self->runCmd("$cmd name $ptrs{name}");
 	}
 	# PROTO
 	if (defined $ptrs{proto} && $ptrs{proto} ne "" && $ptrs{proto} ne $policy{proto}) {
 		$self->runCmd("$cmd target proto $ptrs{proto}");
 	} elsif (defined $ptrs{proto} && $ptrs{proto} ne "" && $policy{proto} ne $ptrs{proto}) {
 		$self->runCmd("$cmd no target proto");
 	}
 	# TOS
 	if (defined $ptrs{tos} && $ptrs{tos} ne "" && $ptrs{tos} ne $policy{tos}) {
 		$self->runCmd("$cmd target tos $ptrs{tos}");
 	} elsif (defined $ptrs{tos} && $ptrs{tos} eq "" && $ptrs{tos} ne $policy{tos}) {
 		$self->runCmd("$cmd no target tos");
 	}
 	# PORT
 	if (defined $ptrs{port} && $ptrs{port} ne "" && $ptrs{port} ne $policy{port}) {
 		$self->runCmd("$cmd target port $ptrs{port}");
 	} elsif (defined $ptrs{port} && $ptrs{port} eq "" && $ptrs{port} ne $policy{port}) {
 		$self->runCmd("$cmd no target port");
 	}
 	# UNITS 
 	if (defined $ptrs{units} && $ptrs{units} ne "" && $ptrs{units} ne $policy{units}) {
 		$self->runCmd("$cmd target units oid $ptrs{units}");
 	} elsif (defined $ptrs{units} && $ptrs{units} eq "" && $ptrs{units} ne $policy{units}) {
 		$self->runCmd("$cmd no target units oid");
 	}
 	# FILE
 	if (defined $ptrs{file} && $ptrs{file} ne "" && $ptrs{file} ne $policy{file}) {
 		$self->runCmd("$cmd target file $ptrs{file}");
 	} elsif (defined $ptrs{file} && $ptrs{file} eq "" && $ptrs{file} ne $policy{file}) {
 		$self->runCmd("$cmd no target file");
 	}
 	# ADDRESS
 	if (defined $ptrs{addr} && $ptrs{addr} ne "" && $ptrs{addr} ne $policy{addr}) {
 		$self->runCmd("$cmd target addr $ptrs{addr}");
 	} elsif (defined $ptrs{addr} && $ptrs{addr} eq "" && $ptrs{addr} ne $policy{addr}) {
 		$self->runCmd("$cmd no target addr");
 	}
 	# IFINDEX
 	if (defined $ptrs{ifindex} && $ptrs{ifindex} ne "" && $ptrs{ifindex} ne $policy{ifindex}) {
 		$self->runCmd("$cmd target ifindex $ptrs{ifindex}");
 	} elsif (defined $ptrs{ifindex} && $ptrs{ifindex} eq "" && $ptrs{ifindex} ne $policy{ifindex}) {
 		$self->runCmd("$cmd no target ifindex");
 	}	
 	# POLICY OR
 	if (defined $ptrs{'policy-or'} && $ptrs{'policy-or'} ne "" && $ptrs{'policy-or'} ne $policy{'policy-or'}) {
 		$self->runCmd("$cmd target policy-or $ptrs{'policy-or'}");
 	} elsif (defined $ptrs{'policy-or'} && $ptrs{'policy-or'} eq "" && $ptrs{'policy-or'} ne $policy{'policy-or'}) {
 		$self->runCmd("$cmd no target policy-or");
 	}
 	# POLICY AND
 	if (defined $ptrs{'policy-and'} && $ptrs{'policy-and'} ne "" && $ptrs{'policy-and'} ne $policy{'policy-and'}) {
 		$self->runCmd("$cmd target policy-and $ptrs{'policy-and'}");
 	} elsif (defined $ptrs{'policy-and'} && $ptrs{'policy-and'} eq "" && $ptrs{'policy-and'} ne $policy{'policy-and'}) {
 		$self->runCmd("$cmd no target policy-and");
 	}
 	# TIME
 	if (defined $ptrs{time} && $ptrs{time} ne "" && $ptrs{time} ne $policy{time}) {
 		$self->runCmd("$cmd target time $ptrs{time}");
 	} elsif (defined $ptrs{time} && $ptrs{time} eq "" && $ptrs{time} ne $policy{time}) {
 		$self->runCmd("$cmd no target time");
 	}	
 	# DAY
 	if (defined $ptrs{day} && $ptrs{day} ne "" && $ptrs{day} ne $policy{day}) {
 		$self->runCmd("$cmd target day $ptrs{day}");
 	} elsif (defined $ptrs{day} && $ptrs{day} eq "" && $ptrs{day} ne $policy{day}) {
 		$self->runCmd("$cmd no target day");
 	}
 	# BW
 	if (defined $ptrs{bw_in} && $ptrs{bw_in} ne "" && defined $ptrs{bw_out} && $ptrs{bw_out} ne "") {
 		$self->runCmd("$cmd bw $ptrs{bw_in} in $ptrs{bw_out} out");
 	} elsif (defined $ptrs{day} && $ptrs{day} eq "" && $ptrs{day} ne $policy{day}) {
 		$self->runCmd("$cmd bw 0 in 0 out");
 	} 	 	
 	$self->runCmd("save");
 	return $ptrs{oid};
}

# @lines = runCmd ( command )
# send to NeTAMS "command" and returns output as array
#-------------------------------------------------------------------
sub runCmd {
	my ($self, $result, @lines);
		$self = shift;
		@lines = $self->{nth}->cmd("$_[0]");
	return @lines;
}

#-------------------------------------------------------------------
sub getUnitTypes {
	return qw(host group user net cluster);
}

#-------------------------------------------------------------------
sub getStorageTypes {
	return qw(hash mysql postgres oracle radius);
}

#-------------------------------------------------------------------
sub getDataSourceTypes {
	return qw(ip-traffic netflow libpcap netgraph);
}

#-------------------------------------------------------------------
sub getDataSources {
	return qw(tee divert ipq ulog A.B.C.D ifname nodename);
}

#-------------------------------------------------------------------
sub getPolicyTargets {
	return qw(proto ports tos units file addr ifindex policy time day);
}

#-------------------------------------------------------------------
sub getDebugTypes {
	return qw(none command parse sleep server proc_mux ds_ip storage alert sheduler html monitor login quota iptree flow ds_mux memory policy billing all);
}

# unitLogin ( oid pass ip mac)
#-------------------------------------------------------------------
sub unitLogin {
	my ($self, %ptrs, @out);
		$self = shift;
		%ptrs = (@_);
	$self->runCmd("service login");
	@out = $self->runCmd("login oid $ptrs{oid} password $ptrs{pass} ip $ptrs{ip}");
	$self->runCmd("exit");
	return 1 if $out[0] =~ m/parse: OK:/;
	return 0;
}

# unitLogout ( oid pass ip mac )
#-------------------------------------------------------------------
sub unitLogout {
	my ($self, %ptrs, @out);
		$self = shift;
		%ptrs = (@_);
	$self->runCmd("service login");
	@out = $self->runCmd("logout oid $ptrs{oid} password $ptrs{pass} ip $ptrs{ip}");
	$self->runCmd("exit");
	return 1 if $out[0] =~ m/parse: OK:/;
	return 0;		
}

# %list = getUnitPolicies ( oid , fw|acct , all|custom|default )
#-------------------------------------------------------------------
sub getUnitPolicies {
	my ($self, %proc, @list);
		$self = shift;
	if ($_[1] eq "fw") {
		if ($_[2] =~ m/all|default/) {
			push @list, $_ foreach (split " ", $self->{config}{services}{processor}{'def-fw-policy'});
		}
		if ($_[2] =~ m/all|custom/) {
			push @list, $_ foreach (split " ", $self->{config}{units}{$_[0]}{'fw-policy'});
		}
	} elsif ($_[1] eq "acct") {
		if ($_[2] =~ m/all|default/) {
			push @list, $_ foreach (split " ", $self->{config}{services}{processor}{'def-acct-policy'});
		}
		if ($_[2] =~ m/all|custom/) {
			push @list, $_ foreach (split " ", $self->{config}{units}{$_[0]}{'acct-policy'});
		}
	}
	return @list;	
}


# setServiceProcessor ( param=>value, ...) 
# sets options for service processor
#-------------------------------------------------------------------
sub setServiceProcessor {
	my ($self, %ptrs, @out);
		$self = shift;
		%ptrs = (@_);
	$self->runCmd("service processor");
	
	$self->runCmd("exit");
}


# parses output of "show config" command onto hash
#-------------------------------------------------------------------
sub parseConfig {
	my ($self, %result, @lines, @values, $tmp, %units, %cfg, $cursrv, %re);
		$self = shift;
		# Precompiling big regular expressions for more speed
		$re{version} 		= qr/NeTAMS version (\d\.\d\.\d) (\(.+\))/;
		$re{service} 		= qr/^service ([\w-]+ ?(\d)?)$/;
		$re{user} 			= qr/^user oid (\w{6}) name (\w+)/;
		$re{autoUnit} 		= qr/^auto-units (\d+) type (host|user) naming (by-dns|prefix1|prefix2) ([\w_-]+) group ([\w_-]+)?/;
		$re{unitBW} 		= qr/bw(?: (\d+(?:\.\d{3})?\w?) ?(in|out)?)+/;
		$re{unitAcctPolicy} = qr/acct-policy(( [!%]{0,2}\w{6})+)(?: $| fw-policy| ds-list)/;
		$re{unitFwPolicy} 	= qr/fw-policy ([\w \!\%]+)(?:$| ds-list)/;
		$re{unitIp} 		= qr/ip (\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}(\/\d{1,2})?)/;
		$re{clusterIp} 		= qr/(( ip \d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}(\/\d{1,2})?)+)/;
		$re{unitNetmask}	= qr/mask (\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})/;
		$re{subplan_inc1}	= qr/included (\d+(?:\.\d+\w)?|unlimited) in (\d+(?:\.\d+\w)?|unlimited) out/;
		$re{subplan_inc2}	= qr/included (\d+(?:\.\d+\w)?|unlimited) sum/;
		$re{subplan_fee}	= qr/fee (\d+\.\d+) spread (monthly|hourly|daily)/;
		$re{subplan_over1}	= qr/overdraft (\d+\.\d+) in (\d+\.\d+) out/;
		$re{subplan_over2}	= qr/overdraft (\d+\.\d+) sum/;
		
		# ONLY ENG NAMES:
		$re{policy} 		= qr/^policy oid (\w{6}) name ([\w\-]+) target (.+)/;
		$re{unit} 			= qr/^unit (\w+) oid (\w{6}) name (\S+)/;
		# WITH RUS NAMES:
		#$re{unit} 			= qr/^unit (\w+) oid (\w{6}) name ([\xC0-\xFF|\w_-]+)/;
		#$re{policy} 		= qr/^policy oid (\w{6}) name ([\xC0-\xFF|\w_-]+)( target (.+))?/;
		
	@lines = $self->{nth}->cmd("show config oids");
	$cursrv = 'main';
	foreach (@lines) {
		# Version check
		if ($_ =~ m/$re{version}/) {
			$cfg{version} = $1." ".$2;
		# Service name check
		} elsif ($_ =~ m/$re{service}/) {
			$cfg{services}{$1};
			$cfg{services}{$1}{id} = $2; #if defined $2;
			$cursrv = $1;
		} else {
			# Parsing service related options
			# MAIN
			if ($cursrv =~ m/main/) { 
				# system users
				if ($_ =~ m/$re{user}/) {
					my $oid = $1;
					$cfg{services}{$cursrv}{users}{$oid}{name} = $2;
					if ($_ =~ m/real-name "(.+)"/) {
						$cfg{services}{$cursrv}{users}{$oid}{'real-name'} = $1;
					} if ($_ =~ m/crypted ([^ \s]+) /) {
						$cfg{services}{$cursrv}{users}{$oid}{password} = $1;
					} if ($_ =~ m/email ([\w_-]+@[\w_\.-]+)/) {
						$cfg{services}{$cursrv}{users}{$oid}{email} = $1;
					} if ($_ =~ m/permit (none|web|all)/) {
						$cfg{services}{$cursrv}{users}{$oid}{permit} = $1;
					}
				} elsif ($_ =~ m/^debug ([\w ]+)/) {
					$cfg{services}{$cursrv}{debug}	= $1;
				} elsif ($_ =~ m/^language (\w{2})/) {
					$cfg{services}{$cursrv}{language}	= $1;
				}
			# SCHEDULER
			} elsif ($cursrv =~ m/^scheduler/) {
				if ($_ =~ m/^oid (\w{6}) time ([\w\-]+) action "(.+)"/) {
					$cfg{services}{$cursrv}{$1}{time} = $2;
					$cfg{services}{$cursrv}{$1}{action} = $3;
				} 
			# SERVER
			} elsif ($cursrv =~ m/server/) {
				my @arr = split(" ",$_);
				$cfg{services}{$cursrv}{$arr[0]} = $arr[1];
			# PROCESSOR				
			} elsif ($cursrv =~ m/processor/) {
				if ($_ =~ m/^lookup-delay (\d+)/) {
					$cfg{services}{$cursrv}{'lookup-delay'} = $1;	
				} elsif ($_ =~ m/flow-lifetime (\d+)/) {
					$cfg{services}{$cursrv}{'flow-lifetime'} = $1;	
				}
				# AUTO UNITS
				if ($_ =~ m/$re{autoUnit}/) {
					$cfg{services}{$cursrv}{'auto-units'}{$1}{type} = $2;
					$cfg{services}{$cursrv}{'auto-units'}{$1}{naming} = $3;
					$cfg{services}{$cursrv}{'auto-units'}{$1}{prefix} = $4;
					$cfg{services}{$cursrv}{'auto-units'}{$1}{group} = $5;
				}
				# UNITS
				if ($_ =~ m/$re{unit}/) {
					my $oid = $2;
					$cfg{units}{$oid}{type} = $1;
					$cfg{groups}{$oid} = $3 if $1 eq "group";
					$cfg{units}{$oid}{name} = $3;
					$cfg{units}{name2oid}{$3} = $oid;
					$cfg{name2oid}{unit}{$3} = $oid;
					if ($_ =~ m/parent (\w{6})/) {
						$cfg{units}{$oid}{parent} = $1;
						$cfg{units}{$1}{childs} .= "$oid,";
					}
					if ($_ =~ m/no-local-pass/) {
						$cfg{units}{$oid}{nolp} = 1;
					}
					if ($_ =~ m/email (\S+)/) {
						$cfg{units}{$oid}{email} = $1;
					}
					if ($_ =~ m/$re{unitBW}/) {
						if ($2 =~ m/(in|out)/) {
							$cfg{units}{$oid}{'bw-in'} = $1 if $_ =~ m/(\d+(?:\.\d{3}\w?)) in/;
							$cfg{units}{$oid}{'bw-out'} = $1 if $_ =~ m/(\d+(?:\.\d{3}\w)?) out/;
						} else {
							$cfg{units}{$oid}{'bw-in'} = $2;
							$cfg{units}{$oid}{'bw-out'} = $2;
						}
					} 
					if ($_ =~ m/$re{unitAcctPolicy}/) {
						$cfg{units}{$oid}{'acct-policy'} = $1;
						#@{$cfg{units}{$oid}{acctpols}} = split / /, $1;
					}
					if ($_ =~ m/$re{unitFwPolicy}/) {
						$cfg{units}{$oid}{'fw-policy'} = $1;
					}
					if ($_ =~ m/ds-list ([\d\,]+)/) {
						$cfg{units}{$oid}{'ds-list'} = $1;
					}
					if ($cfg{units}{$oid}{type} eq "cluster" && $_ =~ m/$re{clusterIp}/) {
						$cfg{units}{$oid}{'ip'} = $1;
						$cfg{units}{$oid}{'ip'} =~ s/ip //g;
						$cfg{units}{$oid}{'ip'} =~ s/^ //g;
					} elsif ($_ =~ m/$re{unitIp}/) {
						$cfg{units}{$oid}{'ip'} = $1;
					}
					if ($_ =~ m/$re{unitNetmask}/) {
						$cfg{units}{$oid}{'mask'} = $1;
					}
					if ($_ =~ m/mac (\w{2}:\w{2}:\w{2}:\w{2}:\w{2}:\w{2})/) {
						$cfg{units}{$oid}{'mac'} = $1;
					}					
					if ($_ =~ m/password (\w+)/) {
						$cfg{units}{$oid}{password} = $1;
					}
					if ($_ =~ m/description "(.+)"/ or $_ =~ m/description (\S+)/) {
						$cfg{units}{$oid}{description} = $1;
					}					
					if ($cfg{units}{$oid}{type} eq "net" && $_ =~ m/auto-units (\d+)/) {
						$cfg{units}{$oid}{'auto-units'} = $1;
					}
				# Policies
				} elsif ($_ =~ m/$re{policy}/) {
					my $oid = $1;
					my $targ = $3;
					$cfg{policies}{$oid}{name} = $2;
					$cfg{policies}{$oid}{target} = $3;
					$cfg{policies}{name2oid}{$2} = $oid;
					if ($targ =~ m/proto (\w+)/) {
						$cfg{policies}{$oid}{proto} = $1;
					}
					if ($targ =~ m/tos (\d+)/) {
						$cfg{policies}{$oid}{tos} = $1;
					}
					if ($targ =~ m/port (([bsd]?[\d -:]+)+)/) {
						$cfg{policies}{$oid}{port} = $1;
					}
					if ($targ =~ m/units oid (\w{6})/) {
						$cfg{policies}{$oid}{units} = $1;
					}
					if ($targ =~ m/file ([\w\.\/\-\_]+)/) {
						$cfg{policies}{$oid}{file} = $1;
					}
					if ($targ =~ m/addr (([bsd]?\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3} )+)/) {
						$cfg{policies}{$oid}{addr} = $1;
					}
					if ($targ =~ m/ifindex (([bsd]?\d+ )+)/) {
						$cfg{policies}{$oid}{ifindex} = $1;
					}
					if ($targ =~ m/policy-(or|and) ([\w !]+)/) {
						$cfg{policies}{$oid}{'policy-'.$1} = $2;
					}
					if ($targ =~ m/time (\d{2}:\d{2}-\d{2}:\d{2})/) {
						$cfg{policies}{$oid}{time} = $1;
					}
					if ($targ =~ m/day (\w{3}-\w{3}|\w{3})/) {
						$cfg{policies}{$oid}{day} = $1;
					}
					if ($targ =~ m/bw(?: (\d+(?:\.\d{3})?\w?) ?(in|out)?)+/) {
						if ($2 =~ m/(in|out)/) {
							$cfg{policies}{$oid}{bw_in} = $1 if $_ =~ m/(\d+(?:\.\d{3}\w?)) in/;
							$cfg{policies}{$oid}{bw_out} = $1 if $_ =~ m/(\d+(?:\.\d{3}\w)?) out/;
						} else {
							$cfg{policies}{$oid}{bw_in} = $2;
							$cfg{policies}{$oid}{bw_out} = $2;
						}						
					}
				# Default policies
				} elsif ($_ =~ m/default (acct-policy|fw-policy) ([\w-%! ]+)/) {
					$cfg{services}{$cursrv}{'def-'.$1} = $2;
				} elsif ($_ =~ m/restrict all (pass|drop) local (pass|drop)/) {
					$cfg{services}{$cursrv}{"restrict-all"} = $1;
					$cfg{services}{$cursrv}{"restrict-local"} = $2;
				} elsif ($_ =~ m/access-script "(.+)"/) {
					$cfg{services}{$cursrv}{'access-script'} = $1;
				} elsif ($_ =~ m/auto-assign ([\d\.]+) ([\d\.]+)/) {
					$cfg{services}{$cursrv}{'auto-assign'} .= "$1-$2 ";
				} 
			# STORAGE
			} elsif ($cursrv =~ m/storage/) {
				if ($_ =~ m/type (\w+)/) {
					$cfg{services}{$cursrv}{type} = $1;
				} elsif ($_ =~ m/^user (\w+)/) {
					$cfg{services}{$cursrv}{user} = $1;
				} elsif ($_ =~ m/^password (\w+)/) {
					$cfg{services}{$cursrv}{password} = $1;
				} elsif ($_ =~ m/^dbname (\w+)/) {
					$cfg{services}{$cursrv}{dbname} = $1;
				} elsif ($_ =~ m/^host (\w+)/) {
					$cfg{services}{$cursrv}{host} = $1;
				} elsif ($_ =~ m/^accept (.+)$/) {
					$cfg{services}{$cursrv}{accept} = $1;
				} elsif ($_ =~ m/^socket (.+)$/) {
					$cfg{services}{$cursrv}{socket} = $1;
				} elsif ($_ =~ m/^port (\d+)$/) {
					$cfg{services}{$cursrv}{port} = $1;
				} elsif ($_ =~ m/^retry (\d+)$/) {
					$cfg{services}{$cursrv}{retry} = $1;
				} elsif ($_ =~ m/^timeout (\d+)$/) {
					$cfg{services}{$cursrv}{timeout} = $1;
				}elsif ($_ =~ m/^nas-ip (\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})/) {
					$cfg{services}{$cursrv}{nas_ip} = $1;
				}
			# DATA-SOURCE	
			} elsif ($cursrv =~ m/data-source/) {
				if ($_ =~ m/type ([\w-]+)/) {
					$cfg{services}{$cursrv}{type} = $1;	
				} elsif ($_ =~ m/source ([\w- \.]+)/) {
					$cfg{services}{$cursrv}{source} .= $1." ";
				} elsif ($_ =~ m/listen ([\w \.]+)/) {
					$cfg{services}{$cursrv}{listen} = $1;
				} elsif ($_ =~ m/clock (remote|local)/) {
					$cfg{services}{$cursrv}{clock} = $1;	
				} elsif ($_ =~ m/rule ([\w- ]+)/) {
					$cfg{services}{$cursrv}{rule} = $1;
				}
			# ALERTER
			} elsif ($cursrv =~ m/alerter/) {
				if ($_ =~ m/report oid (\w{6}) name (\w+)/) {
					$cfg{services}{$cursrv}{report}{$1}{name} = $2;
				} elsif ($_ =~ m/smtp-server (\S+)/) {
					$cfg{services}{$cursrv}{'smtp-server'} = $1;
				}
			# HTML
			} elsif ($cursrv =~ m/html/) {
				if ($_ =~ m/language (en|ru)/) {
					$cfg{services}{$cursrv}{language} = $1;	
				} elsif ($_ =~ m/run ([\w+-]+)/) {
					$cfg{services}{$cursrv}{run} = $1;				
				} elsif ($_ =~ m/path (\S+)/) {
					$cfg{services}{$cursrv}{path} = $1;				
				} elsif ($_ =~ m/url (\S+)/) {
					$cfg{services}{$cursrv}{url} = $1;				
				} elsif ($_ =~ m/servlet-url (\S+)/) {
					$cfg{services}{$cursrv}{'servlet-url'} = $1;				
				} elsif ($_ =~ m/htaccess (yes|no)/) {
					$cfg{services}{$cursrv}{htaccess} = $1;				
				} elsif ($_ =~ m/client-pages (all|groups|none)/) {
					$cfg{services}{$cursrv}{'client-pages'} = $1;				
				} elsif ($_ =~ m/account-pages (all|none)/) {
					$cfg{services}{$cursrv}{'account-pages'} = $1;				
				}
			# MONITOR
			} elsif ($cursrv =~ m/monitor/) {
				if ($_ =~ m/monitor to (file|storage) ([\S ]+)/) {
					$cfg{services}{$cursrv}{'type'} = $1;
					$cfg{services}{$cursrv}{'targ'} = $2;				
				} elsif ($_ =~ m/monitor unit (\w{6})/) {
					$cfg{services}{$cursrv}{units} .= "$1 ";				
				}
			# QUOTA
			} elsif ($cursrv =~ m/quota/) {
				if ($_ =~ m/policy (\S+)/) {
					$cfg{services}{$cursrv}{policy} = $1;				
				} elsif ($_ =~ m/soft-treshold (\d+)/) {
					$cfg{services}{$cursrv}{'soft-treshold'} = $1;
				} elsif ($_ =~ m/delay (\d+)/) {
					$cfg{services}{$cursrv}{delay} = $1;				
				} elsif ($_ =~ m/block-policy (.+)/) {
					$cfg{services}{$cursrv}{'block-policy'} = $1;				
				}
			# LOGIN
			} elsif ($cursrv =~ m/login/) {
				if ($_ =~ m/default-inact (\d+)/) {
					$cfg{services}{$cursrv}{'default-inact'} = $1;				
				} elsif ($_ =~ m/default-abs (\d+)/) {
					$cfg{services}{$cursrv}{'default-abs'} = $1;				
				} elsif ($_ =~ m/max-inact (\d+)/) {
					$cfg{services}{$cursrv}{'max-inact'} = $1;				
				} elsif ($_ =~ m/min-inact (\d+)/) {
					$cfg{services}{$cursrv}{'min-inact'} = $1;				
				} elsif ($_ =~ m/max-abs (\d+)/) {
					$cfg{services}{$cursrv}{'max-abs'} = $1;				
				} elsif ($_ =~ m/min-abs (\d+)/) {
					$cfg{services}{$cursrv}{'min-abs'} = $1;				
				} elsif ($_ =~ m/min_passwd_length (\d+)/) {
					$cfg{services}{$cursrv}{'min_passwd_length'} = $1;				
				} elsif ($_ =~ m/delay (\d+)/) {
					$cfg{services}{$cursrv}{delay} = $1;				
				} elsif ($_ =~ m/set-user-ip (yes|no|1|0)/) {
					$cfg{services}{$cursrv}{'set-user-ip'} = $1;				
				} elsif ($_ =~ m/storage (\d+)/) {
					$cfg{services}{$cursrv}{storage} = $1;
				} elsif ($_ =~ m/relogin (yes|no|1|0)/) {
					$cfg{services}{$cursrv}{relogin} = $1;
				}
			# BILLING
			} elsif ($cursrv =~ m/billing/) {
				# subplan
				if ($_ =~ m/^subplan (\d+)/) {
					my $id = $1;
					if ($_ =~ m/$re{subplan_fee}/) {
						$cfg{services}{$cursrv}{subplan}{$id}{fee} = $1;				
						$cfg{services}{$cursrv}{subplan}{$id}{spread} = $2;				
					} elsif ($_ =~ m/$re{subplan_inc1}/) {
						$cfg{services}{$cursrv}{subplan}{$id}{incl_in} = $1;
						$cfg{services}{$cursrv}{subplan}{$id}{incl_out} = $2;
					} elsif ($_ =~ m/$re{subplan_inc2}/) {
						$cfg{services}{$cursrv}{subplan}{$id}{incl_sum} = $1;
					} elsif ($_ =~ m/policy (\S+)/) {
						$cfg{services}{$cursrv}{subplan}{$id}{policy} = $1;				
					} elsif ($_ =~ m/$re{subplan_over1}/) {
						$cfg{services}{$cursrv}{subplan}{$id}{over_in} = $1;
						$cfg{services}{$cursrv}{subplan}{$id}{over_out} = $2;
					} elsif ($_ =~ m/$re{subplan_over2}/) {
						$cfg{services}{$cursrv}{subplan}{$id}{over_sum} = $1;
					} elsif ($_ =~ m/adjust-included (yes|no)/) {
						$cfg{services}{$cursrv}{subplan}{$id}{adjust_included} = $1;
					} elsif ($_ =~ m/adjust-fee (yes|no)/) {
						$cfg{services}{$cursrv}{subplan}{$id}{adjust_fee} = $1;
					}
				# plan
				} elsif ($_ =~ m/^plan (\d+)/) {
					my $id = $1;
					if ($_ =~ m/^plan \d+ name (\w+)/) {
						$cfg{services}{$cursrv}{plan}{$id}{name} = $1;
					} elsif ($_ =~ m/description "(.+)"/) {
						$cfg{services}{$cursrv}{plan}{$id}{descr} = $1;				
					} elsif ($_ =~ m/subplan ([\d ]+)/) {
						$cfg{services}{$cursrv}{plan}{$id}{subplan} = $1;				
					}
				# options
				} elsif ($_ =~ m/delay (\d+)/) {
					$cfg{services}{$cursrv}{delay} = $1;				
				} elsif ($_ =~ m/storage (\d) ?(all|plans|subplans)?/) {
					$cfg{services}{$cursrv}{storage}{$1} = $2;
					$cfg{services}{$cursrv}{storage}{$1} = "all";
				} elsif ($_ =~ m/default-credit-limit (-?\d+\.\d+)/) {
					$cfg{services}{$cursrv}{'default-credit-limit'} = $1;
				}
			# ACL_SERVER
			} elsif ($cursrv =~ m/acl-server/) {
				if ($_ =~ m/^hostname (.+)/) {
					$cfg{services}{$cursrv}{hostname} = $1;
				} elsif ($_ =~ m/^direction (src|dst)/) {
					$cfg{services}{$cursrv}{direction} = $1;
				} elsif ($_ =~ m/^dynamic-name (\w+)/) {
					$cfg{services}{$cursrv}{'dynamic-name'} = $1;
				} elsif ($_ =~ m/^acl-number (\w+)/) {
					$cfg{services}{$cursrv}{'acl-number'} = $1;
				} elsif ($_ =~ m/^delay (\w+)/) {
					$cfg{services}{$cursrv}{'delay'} = $1;
				} elsif ($_ =~ m/^set-uptime (\w+)/) {
					$cfg{services}{$cursrv}{'set-uptime'} = $1;
				}
			}
		}
	}
	
	# Sys-policies
	@lines = $self->{nth}->cmd("show units syspolicy whereset");
	shift @lines;
	foreach (@lines) {
		$_ =~ s/ +//g;
		$_ =~ s/\|/ /g;
		@values = split(" ",$_);
		if ($values[0] =~ m/\w{6}/) {
			$cfg{units}{$values[0]}{'sys-policy'} = $values[2];
		}
	}
	
	# Quota
	@lines = $self->{nth}->cmd("show quota list");
	shift @lines;	
	my @arr = split(" ",$lines[0]);
	for (my $i = -1; $i <= $#arr; $i += 2) { 
		$cfg{units}{$arr[$i]}{quota} = 1;
	}
	
	# Accounts
	@lines = $self->{nth}->cmd("show account");
	my $accId;
	foreach (@lines) {
		if ($_ =~ m/^Name (.+) \((\w{6})\) (UNBLOCKED|BLOCKED) (SYNC|NOTSYNC) bal: (-?\d+\.\d+) cred_lim: (-?\d+\.\d+) plan: ([\w-_]+)/) {
			$accId = $2;
			$cfg{name2oid}{account}{$1}		= $accId;
			$cfg{accounts}{$accId}{name} 	= $1;
			$cfg{accounts}{$accId}{status} 	= $3;
			$cfg{accounts}{$accId}{balance} = $5;
			$cfg{accounts}{$accId}{credit} 	= $6;
			$cfg{accounts}{$accId}{plan} 	= $7;
		} elsif ($_ =~ m/Units: (.+)/) {
			my $unts = $1;
			$unts =~ s/[\w-_]+ ([A-F\d]{6}) /$1,/g;
			$cfg{accounts}{$accId}{units} = $unts;
		} 
	}	
	$self->{config} = \%cfg;
}

# getOidByName ( type, name ) 
#-------------------------------------------------------------------
sub getOidByName {
	my ($self,$type,$name);
	$self = shift;
	$type = shift;
	$name = shift;
	if ($name ne "") {
		if ($type eq "account") {
			return $self->{config}{name2oid}{account}{$name} if defined $self->{config}{name2oid}{account}{$name};
		} elsif ($type eq "unit") {
			return $self->{config}{name2oid}{unit}{$name} if defined $self->{config}{name2oid}{unit}{$name};
		}
	}
}

# closes connection
#-------------------------------------------------------------------
sub close {
	my ($self);
		$self = shift;
		$self->runCmd("save");
#		$self->runCmd("exit");
		$self->{nth}->close();
	return 1;
}

1;