#!/usr/bin/perl
# $Id$

our $nawtRoot;
our %cfg;
our $netams;

BEGIN {
	$nawtRoot = $ARGV[0] || $ENV{PWD};
	$nawtRoot =~ s/\/sbin//g;
	#$nawtRoot = "/var/www/apache2/addon-modules/nawt"; # uncomment this if you have problems with cron
	unshift (@INC, $nawtRoot."/lib");
}

use strict;
use Netams;
use Parse::PlainConfig;
use DBI;
use Nawt::SQL;

# Loading nawt.conf
my $config = Parse::PlainConfig->new('DELIM' => '=', 'FILE' => $nawtRoot.'/etc/nawt.conf', 'PURGE' => 1);
foreach ($config->directives) {	$cfg{$_} = $config->get($_); }

# Connecting to NeTAMS
$netams = Netams->new(host => $cfg{nahost}, 
				   port => $cfg{naport}, 
				   user => $cfg{nauser}, 
				   pass => $cfg{napass}
				   );
my $res = $netams->connect;
die "Can't connect to NeTAMS. Check your nawt.conf.\n" unless $res == 200;
$netams->parseConfig();

print "\nCleaning Storages...\n";
print "This script find and delete old, unsused data in NeTAMS storages.\n";

cleanAccounts();
cleanQuotas();
cleanEvents();
cleanMonitor();
cleanNawtData();

# Connecting to storages, cleaning old data
#-------------------------------------------------------------------
sub cleanMonitor {
	print "\n  Searching old monitor data...\n";
	my $time = time - 30*24*60*60;
	my $dbh = getDBH('monitor');
	my ($count) = Nawt::SQL->quickArray("SELECT COUNT(*) FROM monitor WHERE time < $time",$dbh);
	Nawt::SQL->write("DELETE FROM events WHERE time < $time",$dbh);
	print "  Deleted $count records.\n";
}

# Connecting to storages, cleaning old data
#-------------------------------------------------------------------
sub cleanEvents {
	print "\n  Searching old events...\n";
	my $time = time - 30*24*60*60;
	my $dbh = getDBH('all');
	my ($count) = Nawt::SQL->quickArray("SELECT COUNT(*) FROM events WHERE time < $time",$dbh);
	Nawt::SQL->write("DELETE FROM events WHERE time < $time",$dbh);
	print "  Deleted $count records.\n";
}

# Connecting to storages, cleaning old data
#-------------------------------------------------------------------
sub cleanAccounts {
	print "\n  Searching old accounts...\n";
	my $dbh = getDBH('billing');
	my %sql = Nawt::SQL->buildHash("SELECT oid,name FROM billing",$dbh);
	my %ams = $netams->getAccount();
	my $count = 0;
	foreach (keys %sql) {
		my $oid = uc sprintf "%06lx",$_;
		if (!defined $ams{$oid}) {
			Nawt::SQL->write("DELETE FROM billing WHERE oid=$_",$dbh);
			print "    Account $sql{$_} ($oid) undefined ... Deleted\n";
			$count++;
		}
	}
	print "  Deleted $count records.\n";
}


# Connecting to storages, cleaning old data
#-------------------------------------------------------------------
sub cleanQuotas {
	print "\n  Searching old quotas...\n";
	my $dbh = getDBH('quota');
	my %sql = Nawt::SQL->buildHash("SELECT unit_oid,policy_oid FROM quota",$dbh);
	my %unit = $netams->getUnit();
	my %policy = $netams->getPolicy();
	my $count = 0;
	foreach (keys %sql) {
		my $oid = uc sprintf "%06lx",$_;
		my $pid = uc sprintf "%06lx",$sql{$_};
		if (!defined $unit{$oid} or !defined $policy{$pid}) {
			Nawt::SQL->write("DELETE FROM quota WHERE unit_oid=$_ AND policy_oid=$sql{$_}",$dbh);
			print "    Bad unit $oid or policy $pid ... Deleted\n";
			$count++;
		}
	}
	print "  Deleted $count records.\n";	
}

#-------------------------------------------------------------------
sub cleanNawtData {
	my ($dbh,%unit,@sql,$count);
	print "\n  Searching old notes...\n";
	$dbh = DBI->connect($cfg{dsn},$cfg{dbuser},$cfg{dbpass},{ RaiseError=>0,AutoCommit=>1 });
	%unit = $netams->getUnit();
	@sql = Nawt::SQL->buildArray("SELECT DISTINCT parentId FROM nawt_note WHERE parentClass LIKE 'unit'",$dbh);
	$count = 0;
	foreach (@sql) {
		if (not exists $unit{$_}) {
			Nawt::SQL->write("DELETE FROM nawt_note WHERE parentClass LIKE 'unit' AND parentId LIKE '$_'",$dbh);
			$count++;
		}
	}
	print "  Deleted $count records.\n";		
}

# returns database handler for specified data-type
# getDBH ( [data_type] )
#-------------------------------------------------------------------
sub getDBH {
	my ($dbh, %conf, $type, $dsn, $user, $pass);
	$type = $_[0];
	%conf = $netams->getService("storage",1);
	
	foreach (keys %conf) {
		if ($conf{$_}{type} =~ m/(mysql|postgres|oracle)/) {
			my $dbtype = $1;
			$dbtype = "Oracle" if $dbtype eq "oracle";
			if ($conf{$_}{accept} =~ /all|$type/) {
				my $dbname = $conf{$_}{dbname} || "netams";
				$dsn = "DBI:$dbtype:$dbname";
				$dsn .= "@".$conf{$_}{host} if $conf{$_}{host} ne "";
				$user = $conf{$_}{user};
				$pass = $conf{$_}{password};
			}
		}
	}

	$dbh = DBI->connect($dsn,$user,$pass,{ RaiseError=>0,AutoCommit=>1 });
	return $dbh;
}

print "\nDone.\n\n";
1;