#!/usr/bin/perl
# $Id: testConfig.pl,v 1.7 2005/04/11 18:03:46 ramirez Exp $

our $nawtRoot;
our $DBH;
our $AMS;

BEGIN {
	$nawtRoot = $ARGV[0] || $ENV{PWD};
	$nawtRoot =~ s/\/sbin//g;
	unshift (@INC, $nawtRoot."/lib");
}

use strict;
#use CPAN;
#use CGI;

# Check if we run script under shell or web server
# Commented, because this function is unstable/untested
#if (!defined $ENV{SHELL}) {
#	my $cgi = CGI->new;
#	print $cgi->header;
#	print "<pre>";
#}

print "\nNaWT is checking your system environment:\n\n";

my ($os, $prereq, $dbi, $dbDrivers, $cpan, @MODS);
	$prereq = 1;
	$cpan = 0;
	@MODS = qw(Net::Telnet Parse::PlainConfig Date::Calc Tie::IxHash Net::Ping);

###################################
# DETECT OS
###################################
if ($^O =~ /^Win/i) {
	$os = "Windows";
} else {
	$os = "Unix ($^O)";
}
print dotLine("Operating system").$os."\n";

###################################
# Checking DIRS
###################################
print dotLine("NaWT Root");
if (-d $nawtRoot && -d $nawtRoot."/lib") {
	print "$nawtRoot\n";
} else {
	print "Not Found ($nawtRoot)\n\tMaybe non-standart location? Then try use: testConfig.pl /your/nawt/path\n\n";
	exit
}

###################################
# Checking Perl
###################################

print dotLine("Perl Interpreter");
if ($] >= 5.8.0) {
	print "OK\n";
} else {
	print "Please upgrade to 5.8 or later!\n";
	print "Test environment exiting, cannot continue without Perl 5.8.\n";
	exit;
}

print "Perl Modules:\n";

print dotLine("  CPAN module (optional)");
if (eval { require CPAN }) {
	eval { use CPAN };
	print "OK\n";
	$cpan = 1;
} else {
	print "Not installed\n\tWith CPAN module, this script can automatically download and install required modules\n\n";
	$cpan = 0;
}

my $dbi;
print dotLine("  DBI module");
if (eval { require DBI }) {
	print "OK\n";
	$dbi = 1;
} else {
	if ($< == 0 && $os =~ /^Unix/i && $cpan == 1) {
        	print "Attempting to install...\n";
                CPAN::Shell->install("DBI");
		eval {require DBI};
		$dbi = 1;
        } else {
                print "Please install.\n";
		$prereq = 0;
		$dbi = 0;
        }
}

print dotLine("  Avalable database drivers");
if ($dbi) {
	print join(", ",DBI->available_drivers);
	$dbDrivers = join(", ",DBI->available_drivers);
} else {
	print "None";
	$prereq = 0;
}
print "\n";

###################################
# Checking Perl Modules
###################################
foreach (@MODS) {
	checkModule($_);	
}

# this is here to insure they installed correctly.
if ($prereq) {
	print "NaWT modules:\n";
	my @nawt = qw(Netams Nawt Nawt::DateTime Nawt::Debug Nawt::Operation Nawt::Privilegies Nawt::Session Nawt::SQL Nawt::Theme);
	foreach (@nawt) {
		checkModule($_);
	}
	
} else {
	print "Cannot continue with errors.\n";
	exit;
}

unless ($prereq) {
	print "Cannot continue without NaWT files.\n";
	exit;
}

my (@files, $file, $dir, $error);
if ($os eq "Windows") {
	$dir = $nawtRoot."\\etc\\";
} else {
	$dir = $nawtRoot."/etc/";
}

opendir (DIR,$dir) or $error = "Can't open etc (".$dir.") directory!";
if ($error ne "") {
	print $error."\nCannot continue.\n";
	exit;
} else {
        @files = readdir(DIR);
        foreach $file (@files) {
			next if $file !~ m/^nawt.conf$/;
			###################################
			# Checking Config File
			###################################

			print "\n";
			print dotLine("Found config file"); print $file."\n";
			print dotLine("Verifying file");
			my ($config);
			$config = Parse::PlainConfig->new('DELIM' => '=',
                		'FILE' => $dir.$file,
                		'PURGE' => 1);
			unless (defined $config) {
				print "Couldn't open the config file.";
				$prereq = 0;
			} elsif ($config->get('dsn') !~ /\DBI\:\w+\:\w+/) {
				print "DSN is improperly formatted.";
				$prereq = 0;
			} else {
				print "OK\n";
			}

			unless ($prereq) {
        		print " Skipping this configuration.\n";
				$prereq = 1;
			} else {

				###################################
				# Checking extras folder
				###################################

				print dotLine("Extras folder");
               	if (-d $config->get('extrasDIR')) {
					print "OK\n";
				} else {
					print "Appears to be missing!\n";
				}

                ###################################
                # NaWT Checking for database driver 
                ###################################

                print dotLine("Database driver");
				my (@driver);
				@driver = split(/:/,$config->get('dsn'));
                if ($dbDrivers =~ m/$driver[1]/) {
                    print "OK\n";
                } else {
                    print "Not installed!\n";
                }

				###################################
				# NaWT Checking database connection
				###################################

				print dotLine("Database connection");
				my ($test);
				unless (eval {$DBH = DBI->connect($config->get('dsn'),$config->get('dbuser'),$config->get('dbpass'))}) {
					print "Can't connect with info provided!\n";
				} else {
					print "OK\n";
					#print dotLine("NaWT tables");
					#$DBH->disconnect();
				}

				###################################
				# Checking Netams
				###################################
				print dotLine("NeTAMS connection");
				my ($nahost, $naport, $nauser, $napass);
					$nahost = $config->get('nahost') || "localhost";
					$naport = $config->get('naport') || "20001";
					$nauser = $config->get('nauser') || "admin";
					$napass = $config->get('napass') || "aaa";
				eval { require Netams };
					$AMS = Netams->new( host => $nahost, port => $naport, user => $nauser, pass => $napass );
				if ($AMS->connect == 200) {
					print "OK $AMS->{options}{version}\n";
				} else {
					print "Can't connect with info provided!\n";
				}
          }
        }
        closedir(DIR);
}


###################################
# Checking Version
###################################

#print "\nLatest version ........................... ";

print "\nTesting complete!\n";
print "\nLooks good.\n\n" if $prereq;


# CHECK PRIVILEGIES
if ($prereq) {
	use Nawt::SQL;
	
	$AMS->parseConfig();
	my %main = $AMS->getService('main');
	my %user = %{$main{users}};
	my $result;
	
	foreach (keys %user) {
		my ($item) = Nawt::SQL->quickArray("SELECT COUNT(*) FROM nawt_premission WHERE userId LIKE '$_' AND action LIKE 'user' AND value='1'",$DBH);
		$result = 1 if $item;
	}	
	
	if (not $result) {
		print "Looks like admin premissions damaged or not set\nPlease, select user to set premissions:\n\n";
		my $i;
		my @arr;
		my $num;
		foreach (keys %user) {
			$i++;
			$arr[$i] = $_;
			print " $i. $user{$_}{name}\n";
		}		
		while (!defined $arr[$num]) {
			print "\nInvalid number!\n" if defined $num;
			print "\nEnter number: ";
			$num = <>;
		}

		Nawt::SQL->write("INSERT INTO nawt_premission SET userId = '$arr[$num]', action = 'user', value= '1'",$DBH);
		print "Premissions set. Now you can login into NaWT with this user\n\n";
	}
	
	$DBH->disconnect();
}
	


###################################
# Some system subs
###################################

#--------------------------------------------------------------------
sub dotLine {
	my ($text, $line, $num);
		$text = $_[0];
		$num = 35 - length($text);
		$line = "."x$num;
	return $text." ".$line." ";			
}

# checkModule ( module_name , try_install? )
#--------------------------------------------------------------------
sub checkModule {
	my	$module = $_[0];
	print dotLine("  ".$module);
	if (eval "require $module") {
        print "OK\n";
	} else {
		if ($< == 0 && $os =~ /^Unix/i && $cpan == 1 && $_[1] != 1) {
        	print "Attempting to install...\n";
            CPAN::Shell->install($module);
    	} else {
            print "Please install.\n";
			$prereq = 0;
    	}
	}
}

