#!/usr/bin/perl
# this script updates ipfw rule set. usefull with ds-type netflow and etc...
# when runs from cron use: ..../update_ipfw.pl /path/to/nawt/dir
# $Id$
##########################################

# ipfw cmd and params
our $fwcmd = "/sbin/ipfw -q";
# interface name
our $iface = "fxp0";

# rule number for units ipis. and action [allow|deny]
our $ruleNum1 = 1125;
our $action1 = "allow";

# useful to add block all from your networks.
our $networksFile = "/var/www/apache2/addon-modules/nawt/our-nets.txt";
our $ruleNum2 = 1126; #rule num for ips from file
our $action2 = "deny"; # action [allow|deny]

#########################################

our $nawtRoot;
our %cfg;
our $netams;
our $dbh;

BEGIN {
	$nawtRoot = $ARGV[0] || $ENV{PWD};
	$nawtRoot =~ s/\/sbin//g;
	#$nawtRoot = "/var/www/apache2/addon-modules/nawt"; # uncomment this if you have problems with cron
	unshift (@INC, $nawtRoot."/lib");
}

use strict;
use Netams;
use Parse::PlainConfig;
use DBI;
use Nawt::SQL;

# Loading nawt.conf
my $config = Parse::PlainConfig->new('DELIM' => '=', 'FILE' => $nawtRoot.'/etc/nawt.conf', 'PURGE' => 1);
foreach ($config->directives) {	$cfg{$_} = $config->get($_); }

# Connecting to NeTAMS
$netams = Netams->new(host => $cfg{nahost}, 
				   port => $cfg{naport}, 
				   user => $cfg{nauser}, 
				   pass => $cfg{napass}
				   );
my $res = $netams->connect;
die "Can't connect to NeTAMS. Check your nawt.conf.\n" unless $res == 200;
$netams->parseConfig();
$dbh = DBI->connect($cfg{dsn},$cfg{dbuser},$cfg{dbpass},{ RaiseError=>0,AutoCommit=>1 }) or die "Can't connect to database.\n";

exit if -f "/tmp/nawt_ipfw.lock";
`touch /tmp/nawt_ipfw.lock`;

update_ipfw();
                                                                                                                                    
print "\nUpdate Finished\n\n"; 
`rm /tmp/nawt_ipfw.lock`;

#------------------------------------------------------------------------------
sub update_ipfw {
	my @lines = `$fwcmd show`;
	my (%ipfw,%units,%nums);
	%units = $netams->getUnit();
	`$fwcmd delete $ruleNum1`;
	# ADD UNITS
	foreach (keys %units) {
		if ($units{$_}{type} =~ m/host|user|cluster/ && $units{$_}{'sys-policy'} !~ m/deny/) {
			my @ips = split " ",$units{$_}{ip};
			foreach (@ips) {
				`$fwcmd add $ruleNum1 $action1 ip from $_ to any`;			
			}
		}
	}
	
	# ADD IPs FROM FILE
	if ($networksFile ne "") {
		open (FH, $networksFile) || die "Can't open networks file: $networksFile";
		`$fwcmd delete $ruleNum2`;
		while (<FH>) {
			$_ =~ s/\n//g;
			if ($_ =~ m/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\/\d+/) {
				`$fwcmd add $ruleNum2 $action2 ip from $_ to any`;
			}
		}
		close (FH);
	}
}
